<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/17/18
 * Time: 4:54 PM
 */

namespace Orbipay\PaymentsApi\Client\Enum;

use BadMethodCallException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use ReflectionException;
use UnexpectedValueException;

abstract class Enum implements \JsonSerializable
{
    protected $value;

    protected static $cache = array();

    /**
     * Enum constructor.
     * @param $value
     * @throws UnexpectedValueException if incompatible type is given.
     */
    public function __construct($value)
    {
        if (!$this->isValidValue($value)) {
            throw new UnexpectedValueException("Value '$value' is not part of the enum " . get_called_class());
        }

        $this->value = $value;
    }

    /**
     * @return mixed
     */
    public function value()
    {
        return $this->value;
    }

    /**
     * Returns the enum key (i.e. the constant name).
     * @return string
     */
    public function name()
    {
        return static::search($this->value);
    }

    /**
     * @return false|int|string
     */
    public function __toString()
    {
        return $this->name();
    }

    /**
     * @param Enum $enum
     * @return bool
     */
    final public function equals(Enum $enum = null)
    {
        return $enum !== null && $this->value() === $enum->value() && get_called_class() == get_class($enum);
    }

    /**
     * Returns the names (keys) of all constants in the Enum class
     * @return array
     */
    public static function names()
    {
        return array_keys(static::getConstList());
    }

    /**
     * Returns instances of the Enum class of all Enum constants
     * @return array
     */
    public static function values()
    {
        $values = array();
        foreach (static::getConstList() as $key => $value) {
            $values[$key] = new static($value);
        }
        return $values;
    }

    /**
     * @return mixed
     */
    public static function getConstList()
    {
        $logger = LoggerFactory::getLogger(Enum::class);
        $class = get_called_class();
        try {
            if (!array_key_exists($class, static::$cache)) {
                $reflection = new \ReflectionClass($class);
                static::$cache[$class] = $reflection->getConstants();
            }
        } catch (ReflectionException $exception) {
            $logger->error($exception->getMessage(), [], $exception);
        }
        return static::$cache[$class];
    }

    /**
     * @param $value
     * @return bool
     */
    public static function isValidValue($value)
    {
        return in_array($value, static::getConstList(), true);
    }

    /**
     * @param string[] $values
     * @param OrbipayApiException|null $exception
     * @param string $fieldName
     * @param string $appErrorCode
     * @return bool
     */
    public static function areValidValues(array $values, OrbipayApiException $exception = null, $fieldName = '', $appErrorCode = '')
    {
        $iteratedValues = array();
        foreach ($values as $value) {
            if (!self::isValidValue($value) || in_array($value, $iteratedValues)) {
                $exception->addFieldErrorAndResponseCodes($fieldName, $appErrorCode);
                return false;
            }
            $iteratedValues[] = $value;
        }
        return true;
    }

    /**
     * @param $key
     * @return bool
     */
    public static function isValidName($key)
    {
        $array = static::getConstList();
        return isset($array[$key]);
    }

    /**
     * @param $value
     * @return false|int|string
     */
    public static function search($value)
    {
        return array_search($value, static::getConstList(), true);
    }

    /**
     * @param $name
     * @param $arguments
     * @return Enum
     */
    public static function __callStatic($name, $arguments)
    {
        $array = static::getConstList();
        if (isset($array[$name])) {
            return new static($array[$name]);
        }
        throw new BadMethodCallException("No static method or enum constant '$name' in class " . get_called_class());
    }

    /**
     * @param $value
     * @return mixed
     */
    public static function fromValue($value)
    {
        if (!static::isValidValue($value)) {
            return null;
        }

        return new static($value);
    }

    /**
     * @return mixed
     */
    public function jsonSerialize()
    {
        return $this->value();
    }
}