<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/19/18
 * Time: 7:12 PM
 */

namespace Orbipay\PaymentsApi\Client\Exception;


use Exception;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Model\Base\ResponseCode;
use Orbipay\PaymentsApi\Client\Model\Error\Error;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;

class OrbipayApiException extends Exception
{

    /**
     * @var Error[]
     */
    private $errors = array();

    /**
     * @var ResponseCode[]
     */
    private $response_codes = array();

    /**
     * @var string
     */
    private $http_response_code;


    /**
     * @var string
     */
    private $http_response_headers;

    /**
     * @var string
     */
    private $masked_client_request;

    /**
     * @var string
     */
    private $masked_http_request;

    /**
     * @var string
     */
    private $masked_authorization_data;

    /**
     * @var string
     */
    private $response_body;

    /**
     * @return string
     */
    public function getResponseBody()
    {
        return $this->response_body;
    }

    /**
     * @return Error[]
     */
    public function getErrors()
    {
        return $this->errors;
    }

    /**
     * @return ResponseCode[]
     */
    public function getResponseCodes()
    {
        return $this->response_codes;
    }

    /**
     * @return string
     */
    public function getHttpResponseCode()
    {
        return $this->http_response_code;
    }

    /**
     * OrbipayApiException constructor.
     * @param string $message
     * @param mixed $client_request
     * @param int $code
     * @param mixed $previous
     */
    public function __construct($message = null, $client_request = null, $code = 0, $previous = null)
    {
        parent::__construct($message, $code, $previous);
        $this->masked_client_request = CommonUtil::get_masked_string($client_request);
    }

    public function addFieldErrorAndResponseCodes($field, $responseCode)
    {
        array_push($this->errors, new Error(RestConstants::ERROR_FIELD_CODE, RestConstants::ERROR_FIELD_MESSAGE, $field));
        array_push($this->response_codes, new ResponseCode($responseCode, RestConstants::ERROR_FIELD_MESSAGE));
    }

    public static function getDefaultException($message = null, $client_request = null, $code = 0, $previous = null)
    {
        $exception = new OrbipayApiException($message, $client_request, $code, $previous);
        $exception->errors[] = Error::getTechDifficultiesError();
        return $exception;
    }

    public static function constructFromHttpResponse(ApiException $apiException = null, $client_request = null)
    {
        $exception = new OrbipayApiException($apiException->getMessage(), $client_request, $apiException->getCode(), $apiException);

        if ($apiException) {
            $exception->http_response_code = $apiException->getCode();
            $exception->http_response_headers = CommonUtil::get_masked_string($apiException->getResponseHeaders());
            $exception->masked_http_request = $apiException->getMaskedHttpRequest();
            $exception->masked_authorization_data = $apiException->getMaskedAuthorizationData();
            $exception->response_body = $apiException->getResponseBody();

            if (is_string($apiException->getResponseBody()) && CommonUtil::isJson($apiException->getResponseBody())) {
                $exception->errors = ResponseWrapper::wrapErrors($apiException);
            }

            if (isset($apiException->getResponseHeaders()[HttpParamConstants::RESPONSE_CODES]) && is_array($apiException->getResponseHeaders()[HttpParamConstants::RESPONSE_CODES])) {
                $response_codes = explode(RestConstants::RESPONSE_CODES_DELIMITER, $apiException->getResponseHeaders()[HttpParamConstants::RESPONSE_CODES][0]);
            } else {
                $response_codes = [];
            }

            if (isset($apiException->getResponseHeaders()[HttpParamConstants::RESPONSE_TEXT]) && is_array($apiException->getResponseHeaders()[HttpParamConstants::RESPONSE_TEXT])) {
                $response_text = explode(RestConstants::RESPONSE_TEXT_DELIMITER, $apiException->getResponseHeaders()[HttpParamConstants::RESPONSE_TEXT][0]);
            } else {
                $response_text = [];
            }

            if (count($response_codes) > 0 && count($response_text) && count($response_codes) === count($response_text)) {
                for ($i = 0; $i < count($response_codes); $i++) {
                    $exception->response_codes[] = new ResponseCode($response_codes[$i], $response_text[$i]);
                }
            }
        }
        return $exception;
    }

    public function getDebugSummary()
    {
        $debugSummary = PHP_EOL . '-------------------------DEBUG-SUMMARY-START-------------------------' . PHP_EOL;

        $debugSummary .= PHP_EOL . 'MESSAGE: ' . $this->getMessage() . PHP_EOL;

        $debugSummary .= PHP_EOL . 'STACK: ' . $this->getTraceAsString() . PHP_EOL;

        $debugSummary .= PHP_EOL . 'RESPONSE_BODY: ' . $this->getResponseBody() . PHP_EOL;

        if ($this->errors !== null) {
            $debugSummary .= PHP_EOL . 'ERRORS:' . CommonUtil::get_masked_string($this->errors) . PHP_EOL;
        }

        if ($this->response_codes !== null) {
            $debugSummary .= PHP_EOL . 'RESPONSE_CODES: ' . CommonUtil::get_masked_string($this->response_codes) . PHP_EOL;
        }

        if ($this->masked_client_request !== null) {
            $debugSummary .= PHP_EOL . 'MASKED_CLIENT_REQUEST: ' . $this->masked_client_request . PHP_EOL;
        }

        if ($this->masked_http_request !== null) {
            $debugSummary .= PHP_EOL . 'MASKED_HTTP_REQUEST: ' . $this->masked_http_request . PHP_EOL;
        }

        if ($this->masked_authorization_data !== null) {
            $debugSummary .= PHP_EOL . 'MASKED_AUTHORIZATION_DATA: ' . $this->masked_authorization_data . PHP_EOL;
        }

        if ($this->http_response_code !== null) {
            $debugSummary .= PHP_EOL . 'HTTP_RESPONSE_CODE: ' . $this->http_response_code . PHP_EOL;
        }

        if ($this->http_response_headers !== null) {
            $debugSummary .= PHP_EOL . 'HTTP_RESPONSE_HEADERS: ' . $this->http_response_headers . PHP_EOL;
        }

        $debugSummary .= '------------------------DEBUG-SUMMARY-END-------------------------' . PHP_EOL;

        return base64_encode($debugSummary);
    }
}