<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/21/18
 * Time: 3:04 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\Customer;

use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Gender;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\Customer\Customer;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccount;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccounts;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\CustomerApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;
use Orbipay\PaymentsApi\Client\Swagger\Model\CreateCustomerAccountRequest;
use Orbipay\PaymentsApi\Client\Swagger\Model\CreateCustomerRequest;

/**
 * @internal
 */
class CreatorCustomerHandler
{
    /**
     * @param Customer $customer
     * @return Customer
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function process(Customer $customer)
    {
        $this->validate($customer);
        return $this->callAPI($customer);
    }

    /**
     * @param Customer $customer
     * @return Customer
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    private function callAPI(Customer $customer)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($customer->getApiEndPoint());
        $configuration->setClientKey($customer->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($customer->getInvocationContext()->getSecret());

        $customerApi = new CustomerApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\CustomerResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $customerApi->createCustomer($customer->getInvocationContext()->getChannel(),
                $customer->getInvocationContext()->getClientKey(),
                $customer->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $customer->getInvocationContext()->getIdempotentRequestKey(),
                $customer->getInvocationContext()->getRequestor(),
                $customer->getInvocationContext()->getRequestorType()->value(),
                $this->getBody($customer),
                RequestWrapper::wrapXOPAYHeaders($customer->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customer, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $customer);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customer, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    private function getBody(Customer $customer)
    {
        $createCustomerRequest = new CreateCustomerRequest();
        $createCustomerRequest->setCustomerAccounts($this->getCustomerAccounts($customer->getCustomerAccounts()));
        $createCustomerRequest->setComments($customer->getComments());
        $createCustomerRequest->setCustomerReference($customer->getCustomerReference());
        $createCustomerRequest->setFirstName($customer->getFirstName());
        $createCustomerRequest->setLastName($customer->getLastName());
        $createCustomerRequest->setMiddleName($customer->getMiddleName());
        if ($customer->getGender() !== null) {
            $createCustomerRequest->setGender($customer->getGender()->value());
        }
        $createCustomerRequest->setDateOfBirth($customer->getDateOfbirth());
        $createCustomerRequest->setSsn($customer->getSsn());
        $createCustomerRequest->setLocale($customer->getLocale());
        $createCustomerRequest->setEmail($customer->getEmail());
        $createCustomerRequest->setRegisteredEmail($customer->getRegisteredEmail());
        $createCustomerRequest->setHomePhone($customer->getHomePhone());
        $createCustomerRequest->setWorkPhone($customer->getWorkPhone());
        $createCustomerRequest->setMobilePhone($customer->getMobilePhone());
        $createCustomerRequest->setAddress(RequestWrapper::wrapAddress($customer->getAddress()));
        $createCustomerRequest->setCustomFields((object)$customer->getCustomFields());

        return $createCustomerRequest;
    }

    /**
     * @param CustomerAccounts $customerAccounts
     * @return array
     */
    private function getCustomerAccounts(CustomerAccounts $customerAccounts = null)
    {
        $customerAccountsRequest = null;
        if ($customerAccounts !== null) {
            if (is_array($customerAccounts->getList())) {
                $customerAccountsRequest = array();
                foreach ($customerAccounts->getList() as $customerAccount) {
                    $customerAccountRequest = $this->getCustomerAccount($customerAccount);
                    if ($customerAccountRequest !== null) {
                        $customerAccountsRequest[] = $customerAccountRequest;
                    }
                }
            }
        }
        return $customerAccountsRequest;
    }

    /**
     * @param CustomerAccount $customerAccount
     * @return CreateCustomerAccountRequest
     */
    private function getCustomerAccount(CustomerAccount $customerAccount = null)
    {
        $customerAccountRequest = null;
        if ($customerAccount !== null) {
            $customerAccountRequest = new CreateCustomerAccountRequest();
            $customerAccountRequest->setAccountHolderName($customerAccount->getAccountHolderName());
            $customerAccountRequest->setNickname($customerAccount->getNickname());
            $customerAccountRequest->setAddress(RequestWrapper::wrapAddress($customerAccount->getAddress()));
            $customerAccountRequest->setCustomerAccountReference($customerAccount->getCustomerAccountReference());
            $customerAccountRequest->setAccountNumber($customerAccount->getAccountNumber());
            $customerAccountRequest->setCurrentBalance($customerAccount->getCurrentBalance());
            $customerAccountRequest->setCurrentStatementBalance($customerAccount->getCurrentStatementBalance());
            $customerAccountRequest->setMinimumPaymentDue($customerAccount->getMinimumPaymentDue());
            $customerAccountRequest->setPastAmountDue($customerAccount->getPastAmountDue());
            $customerAccountRequest->setPaymentDueDate($customerAccount->getPaymentDueDate());
            $customerAccountRequest->setStatementDate($customerAccount->getStatementDate());
            $customerAccountRequest->setCustomFields((object)$customerAccount->getCustomFields());
        }

        return $customerAccountRequest;
    }

    /**
     * @param Customer $customer
     * @throws OrbipayApiException
     */
    private function validate(Customer $customer)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $customer);

        $this->validateCustomer($customer, $exception);
        ValidationUtil::validateInvocationContext($customer->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    private function validateCustomer(Customer $customer, OrbipayApiException $exception)
    {

        ValidationUtil::validate($customer->getComments(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::COMMENTS, AppErrorCodes::COMMENTS_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidComments'));

        ValidationUtil::validate($customer->getCustomerReference(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::CUSTOMER_REFERENCE, AppErrorCodes::CUSTOMER_REFERENCE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerReference'));

        ValidationUtil::validate($customer->getFirstName(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::FIRST_NAME, AppErrorCodes::FIRST_NAME_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidFirstName'));

        ValidationUtil::validate($customer->getLastName(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::LAST_NAME, AppErrorCodes::LAST_NAME_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidLastName'));

        ValidationUtil::validate($customer->getMiddleName(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::MIDDLE_NAME, AppErrorCodes::MIDDLE_NAME_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidMiddleName'));

        ValidationUtil::validate($customer->getGender(), DataType::ENUM, Mandatory::NO, $exception, HttpParamConstants::GENDER, AppErrorCodes::GENDER_INVALID, CommonUtil::getValidationMethod(Gender::class, 'isValidValue'));

        ValidationUtil::validate($customer->getDateOfbirth(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::DATE_OF_BIRTH, AppErrorCodes::DATE_OF_BIRTH_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDOB'));

        ValidationUtil::validate($customer->getSsn(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::SSN, AppErrorCodes::SSN_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidSSN'));

        ValidationUtil::validate($customer->getLocale(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::LOCALE, AppErrorCodes::LOCALE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidLocale'));

        ValidationUtil::validate($customer->getEmail(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::EMAIL, AppErrorCodes::EMAIL_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidEmail'));

        ValidationUtil::validate($customer->getHomePhone(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::HOME_PHONE, AppErrorCodes::HOME_PHONE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidHomePhone'));

        ValidationUtil::validate($customer->getWorkPhone(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::WORK_PHONE, AppErrorCodes::WORK_PHONE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidWorkPhone'));

        ValidationUtil::validate($customer->getMobilePhone(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::MOBILE_PHONE, AppErrorCodes::MOBILE_PHONE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidMobilePhone'));

        ValidationUtil::validate($customer->getAddress(), DataType::OBJECT, Mandatory::NO, $exception, HttpParamConstants::ADDRESS, '', CommonUtil::getValidationMethod(ValidationUtil::class, 'validateCustomerAddress'));

        ValidationUtil::validate($customer->getCustomerAccounts(), DataType::OBJECT, Mandatory::YES, $exception, HttpParamConstants::CUSTOMER_ACCOUNTS, AppErrorCodes::SYSTEM_ERROR, CommonUtil::getValidationMethod(__CLASS__, 'validateCustomerAccounts'));
    }

    public static function validateCustomerAccounts(CustomerAccounts $customerAccounts, OrbipayApiException $exception, $segment = '')
    {
        ValidationUtil::validate($customerAccounts->getList(), DataType::GROUP, Mandatory::YES, $exception, $segment, AppErrorCodes::SYSTEM_ERROR, CommonUtil::getValidationMethod(__CLASS__, 'validateCustomerAccountsList'));
    }

    public static function validateCustomerAccountsList(array $list, OrbipayApiException $exception, $baseSegment = '', $appErrorCode = '')
    {
        if (count($list) === 0) {
            $exception->addFieldErrorAndResponseCodes($baseSegment, $appErrorCode);
            return;
        }

        for ($i = 0; $i < count($list); $i++) {
            $segment = $baseSegment . HttpParamConstants::LEFT_SQUARE_BRACE . $i
                . HttpParamConstants::RIGHT_SQUARE_BRACE;
            ValidationUtil::validate($list[$i], DataType::OBJECT, Mandatory::YES, $exception, $segment, '', CommonUtil::getValidationMethod(__CLASS__, 'validateCustomerAccount'));
        }
    }

    public static function validateCustomerAccount(CustomerAccount $customerAccount, OrbipayApiException $exception, $segment = '')
    {

        $segment .= HttpParamConstants::DOT;

        ValidationUtil::validate($customerAccount->getAccountHolderName(), DataType::STRING, Mandatory::YES, $exception, $segment . HttpParamConstants::ACCOUNT_HOLDER_NAME, AppErrorCodes::CUSTOMER_ACCOUNT_HOLDER_NAME_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAccountHolderName'));

        ValidationUtil::validate($customerAccount->getNickname(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::NICKNAME, AppErrorCodes::CUSTOMER_ACCOUNT_NICKNAME_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidNickname'));

        ValidationUtil::validate($customerAccount->getCustomerAccountReference(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::CUSTOMER_ACCOUNT_REFERENCE, AppErrorCodes::CUSTOMER_ACCOUNT_REFERENCE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountReference'));

        ValidationUtil::validate($customerAccount->getAccountNumber(), DataType::STRING, Mandatory::YES, $exception, $segment . HttpParamConstants::ACCOUNT_NUMBER, AppErrorCodes::CUSTOMER_ACCOUNT_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountNumber'));

        ValidationUtil::validate($customerAccount->getCurrentBalance(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::CURRENT_BALANCE, AppErrorCodes::CURRENT_BALANCE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountBalance'));

        ValidationUtil::validate($customerAccount->getCurrentStatementBalance(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::CURRENT_STATEMENT_BALANCE, AppErrorCodes::CURRENT_STATEMENT_BALANCE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountBalance'));

        ValidationUtil::validate($customerAccount->getMinimumPaymentDue(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::MINIMUM_PAYMENT_DUE, AppErrorCodes::MINIMUM_PAYMENT_DUE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountBalance'));

        ValidationUtil::validate($customerAccount->getPastAmountDue(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::PAST_AMOUNT_DUE, AppErrorCodes::PAST_AMOUNT_DUE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountBalance'));

        ValidationUtil::validate($customerAccount->getPaymentDueDate(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::PAYMENT_DUE_DATE, AppErrorCodes::PAYMENT_DUE_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));

        ValidationUtil::validate($customerAccount->getStatementDate(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::STATEMENT_DATE, AppErrorCodes::STATEMENT_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));

        ValidationUtil::validate($customerAccount->getAddress(), DataType::OBJECT, Mandatory::NO, $exception, $segment . HttpParamConstants::ADDRESS, '', CommonUtil::getValidationMethod(ValidationUtil::class, 'validateCustomerAccountAddress'));
    }

    /**
     * @param null $apiResponse
     * @return Customer
     */
    private function wrapResponse($apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($customerResponse, $statusCode, $headers) = $apiResponse;

        $customer = ResponseWrapper::wrapCustomer($customerResponse);
        $customer->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $customer;
    }
}