<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\Customer;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\Customer\Customer;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\CustomerApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class GetCustomerHandler
{
    /**
     * @param Customer $customer
     * @return Customer
     * @throws OrbipayApiException
     */
    public function process(Customer $customer)
    {
        $this->validate($customer);
        return $this->callAPI($customer);
    }

    /**
     * @param Customer $customer
     * @return Customer
     * @throws OrbipayApiException
     */
    private function callAPI(Customer $customer)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($customer->getApiEndPoint());
        $configuration->setClientKey($customer->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($customer->getInvocationContext()->getSecret());

        $customerApi = new CustomerApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\CustomerResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $customerApi->getCustomer($customer->getInvocationContext()->getChannel(),
                $customer->getInvocationContext()->getClientKey(),
                $customer->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $customer->getInvocationContext()->getIdempotentRequestKey(),
                $customer->getInvocationContext()->getRequestor(),
                $customer->getInvocationContext()->getRequestorType()->value(),
                $customer->getPathParamData()->getIdCustomer(),
                RequestWrapper::wrapXOPAYHeaders($customer->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customer, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $customer);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customer, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    /**
     * @param Customer $customer
     * @throws OrbipayApiException
     */
    private function validate(Customer $customer)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $customer);

        ValidationUtil::validate($customer->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));

        ValidationUtil::validateInvocationContext($customer->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    /**
     * @param array $apiResponse
     * @return Customer
     */
    private function wrapResponse(array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($customerResponse, $statusCode, $headers) = $apiResponse;

        $customer = ResponseWrapper::wrapCustomer($customerResponse);
        $customer->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $customer;
    }
}