<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\Customer;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\Customer\Customers;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\CustomerApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class RetrieveCustomersHandler
{
    /**
     * @param Customers $customers
     * @return Customers
     * @throws OrbipayApiException
     */
    public function process(Customers $customers)
    {
        $this->validate($customers);
        return $this->callAPI($customers);
    }

    /**
     * @param Customers $customers
     * @return Customers
     * @throws OrbipayApiException
     */
    private function callAPI(Customers $customers)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($customers->getApiEndPoint());
        $configuration->setClientKey($customers->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($customers->getInvocationContext()->getSecret());

        $customerApi = new CustomerApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\CustomersResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {

            if (ValidationUtil::isNonEmptyString($customers->getQueryParamData()->getQueryId())) {

                $response = $customerApi->retrieveCustomersPage($customers->getInvocationContext()->getChannel(),
                    $customers->getInvocationContext()->getClientKey(),
                    $customers->getInvocationContext()->getProduct()->value(),
                    TimeUtil::getTimeStamp(),
                    $customers->getInvocationContext()->getIdempotentRequestKey(),
                    $customers->getInvocationContext()->getRequestor(),
                    $customers->getInvocationContext()->getRequestorType()->value(),
                    $customers->getPathParamData()->getIdCustomer(),
                    $customers->getQueryParamData()->getQueryId(),
                    RequestWrapper::wrapXOPAYHeaders($customers->getInvocationContext()->getAttributes()),
                    $customers->getQueryParamData()->getPageSize(),
                    $customers->getQueryParamData()->getToIndex(),
                    $customers->getQueryParamData()->getFromIndex());

            } else {

                $form_params = [];
                $form_params[HttpParamConstants::CUSTOMER_REFERENCE] = $customers->getFormParamData()->getCustomerReference();
                $form_params[HttpParamConstants::SSN] = $customers->getFormParamData()->getSsn();
                $form_params[HttpParamConstants::EMAIL] = $customers->getFormParamData()->getEmail();
                $form_params[HttpParamConstants::ACCOUNT_NUMBER] = $customers->getFormParamData()->getAccountNumber();

                $response = $customerApi->retrieveCustomers($customers->getInvocationContext()->getChannel(),
                    $customers->getInvocationContext()->getClientKey(),
                    $customers->getInvocationContext()->getProduct()->value(),
                    TimeUtil::getTimeStamp(),
                    $customers->getInvocationContext()->getIdempotentRequestKey(),
                    $customers->getInvocationContext()->getRequestor(),
                    $customers->getInvocationContext()->getRequestorType()->value(),
                    RequestWrapper::wrapXOPAYHeaders($customers->getInvocationContext()->getAttributes()),
                    $customers->getQueryParamData()->getPageSize(), $form_params);
            }
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customers, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $customers);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customers, $e->getCode(), $e);
        }

        return $this->wrapResponse($customers, $response);
    }

    /**
     * @param Customers $customers
     * @throws OrbipayApiException
     */
    private function validate(Customers $customers)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $customers);

        ValidationUtil::validate($customers->getFormParamData()->getCustomerReference(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::CUSTOMER_REFERENCE, AppErrorCodes::CUSTOMER_REFERENCE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerReference'));
        ValidationUtil::validate($customers->getFormParamData()->getAccountNumber(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::ACCOUNT_NUMBER, AppErrorCodes::CUSTOMER_ACCOUNT_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountNumber'));
        ValidationUtil::validate($customers->getFormParamData()->getSsn(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::SSN, AppErrorCodes::SSN_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidSSN'));
        ValidationUtil::validate($customers->getFormParamData()->getEmail(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::EMAIL, AppErrorCodes::EMAIL_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidEmail'));


        ValidationUtil::validate($customers->getQueryParamData()->getPageSize(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::PAGE_SIZE, AppErrorCodes::PAGE_SIZE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidPageSize'));

        ValidationUtil::validate($customers->getQueryParamData()->getQueryId(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_ID, AppErrorCodes::QUERY_ID_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidQueryId'));
        ValidationUtil::validate($customers->getQueryParamData()->getToIndex(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::TO_INDEX, AppErrorCodes::TO_INDEX_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidToIndex'));
        ValidationUtil::validate($customers->getQueryParamData()->getFromIndex(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::FROM_INDEX, AppErrorCodes::FROM_INDEX_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidFromIndex'));


        ValidationUtil::validateInvocationContext($customers->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    /**
     * @param Customers $customers
     * @param array $apiResponse
     * @return Customers
     */
    private function wrapResponse(Customers $customers, array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($customerResponse, $statusCode, $headers) = $apiResponse;

        ResponseWrapper::wrapCustomers($customers, $customerResponse);
        $customers->withContext(ResponseWrapper::getInvocationContext($headers, $customers), $customers->getInvocationMode(), $customers->getApiEndPoint());
        return $customers;
    }
}