<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\Customer;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\CustomerAccountStatus;
use Orbipay\PaymentsApi\Client\Enum\CustomerStatus;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Gender;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\Customer\Customer;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccount;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccounts;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\CustomerApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;
use Orbipay\PaymentsApi\Client\Swagger\Model\UpdateCustomerAccountRequest;
use Orbipay\PaymentsApi\Client\Swagger\Model\UpdateCustomerRequest;

class UpdateCustomerHandler
{

    /**
     * @param Customer $customer
     * @return Customer
     * @throws OrbipayApiException
     */
    public function process(Customer $customer)
    {
        $this->validate($customer);
        return $this->callAPI($customer);
    }

    /**
     * @param Customer $customer
     * @return Customer
     * @throws OrbipayApiException
     */
    private function callAPI(Customer $customer)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($customer->getApiEndPoint());
        $configuration->setClientKey($customer->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($customer->getInvocationContext()->getSecret());

        $customerApi = new CustomerApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\CustomerResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $customerApi->updateCustomer($customer->getInvocationContext()->getChannel(),
                $customer->getInvocationContext()->getClientKey(),
                $customer->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $customer->getInvocationContext()->getIdempotentRequestKey(),
                $customer->getInvocationContext()->getRequestor(),
                $customer->getInvocationContext()->getRequestorType()->value(),
                $customer->getPathParamData()->getIdCustomer(),
                $this->getBody($customer),
                RequestWrapper::wrapXOPAYHeaders($customer->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customer, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $customer);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customer, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }


    /**
     * @param Customer|null $customer
     * @return UpdateCustomerRequest
     */
    private function getBody(Customer $customer)
    {
        $updateCustomerRequest = new UpdateCustomerRequest();
        $updateCustomerRequest->setCustomerAccounts($this->getCustomerAccounts($customer->getCustomerAccounts()));
        $updateCustomerRequest->setStatus($customer->getStatus() !== null ? $customer->getStatus()->value() : null);
        $updateCustomerRequest->setComments($customer->getComments());
        $updateCustomerRequest->setFirstName($customer->getFirstName());
        $updateCustomerRequest->setLastName($customer->getLastName());
        $updateCustomerRequest->setMiddleName($customer->getMiddleName());
        $updateCustomerRequest->setGender($customer->getGender() !== null ? $customer->getGender()->value() : null);
        $updateCustomerRequest->setDateOfBirth($customer->getDateOfbirth());
        $updateCustomerRequest->setSsn($customer->getSsn());
        $updateCustomerRequest->setLocale($customer->getLocale());
        $updateCustomerRequest->setEmail($customer->getEmail());
        $updateCustomerRequest->setRegisteredEmail($customer->getRegisteredEmail());
        $updateCustomerRequest->setHomePhone($customer->getHomePhone());
        $updateCustomerRequest->setWorkPhone($customer->getWorkPhone());
        $updateCustomerRequest->setMobilePhone($customer->getMobilePhone());
        $updateCustomerRequest->setAddress(RequestWrapper::wrapAddress($customer->getAddress()));
        $updateCustomerRequest->setCustomFields((object)$customer->getCustomFields());

        return $updateCustomerRequest;
    }

    /**
     * @param CustomerAccounts $customerAccounts
     * @return array
     */
    private function getCustomerAccounts(CustomerAccounts $customerAccounts = null)
    {
        $customerAccountsRequest = null;
        if ($customerAccounts !== null) {
            if (is_array($customerAccounts->getList())) {
                $customerAccountsRequest = array();
                foreach ($customerAccounts->getList() as $customerAccount) {
                    $customerAccountRequest = $this->getCustomerAccount($customerAccount);
                    if ($customerAccountRequest !== null) {
                        $customerAccountsRequest[] = $customerAccountRequest;
                    }
                }
            }
        }
        return $customerAccountsRequest;
    }

    /**
     * @param CustomerAccount $customerAccount
     * @return UpdateCustomerAccountRequest
     */
    private function getCustomerAccount(CustomerAccount $customerAccount = null)
    {
        $updateCustomerAccountRequest = null;
        if ($customerAccount !== null) {
            $updateCustomerAccountRequest = new UpdateCustomerAccountRequest();
            $updateCustomerAccountRequest->setStatus($customerAccount->getStatus() !== null ? $customerAccount->getStatus()->value() : null);
            $updateCustomerAccountRequest->setAccountHolderName($customerAccount->getAccountHolderName());
            $updateCustomerAccountRequest->setNickname($customerAccount->getNickname());
            $updateCustomerAccountRequest->setAddress(RequestWrapper::wrapAddress($customerAccount->getAddress()));
            $updateCustomerAccountRequest->setCustomerAccountReference($customerAccount->getCustomerAccountReference());
            $updateCustomerAccountRequest->setAccountNumber($customerAccount->getAccountNumber());
            $updateCustomerAccountRequest->setCurrentBalance($customerAccount->getCurrentBalance());
            $updateCustomerAccountRequest->setCurrentStatementBalance($customerAccount->getCurrentStatementBalance());
            $updateCustomerAccountRequest->setMinimumPaymentDue($customerAccount->getMinimumPaymentDue());
            $updateCustomerAccountRequest->setPastAmountDue($customerAccount->getPastAmountDue());
            $updateCustomerAccountRequest->setPaymentDueDate($customerAccount->getPaymentDueDate());
            $updateCustomerAccountRequest->setStatementDate($customerAccount->getStatementDate());
            $updateCustomerAccountRequest->setCustomFields((object)$customerAccount->getCustomFields());
        }

        return $updateCustomerAccountRequest;
    }

    /**
     * @param Customer $customer
     * @throws OrbipayApiException
     */
    private function validate(Customer $customer)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $customer);

        ValidationUtil::validate($customer->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));

        $this->validateCustomer($customer, $exception);
        ValidationUtil::validateInvocationContext($customer->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    private function validateCustomer(Customer $customer, OrbipayApiException $exception)
    {
        ValidationUtil::validate($customer->getFirstName(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::FIRST_NAME, AppErrorCodes::FIRST_NAME_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidFirstName'));

        ValidationUtil::validate($customer->getLastName(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::LAST_NAME, AppErrorCodes::LAST_NAME_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidLastName'));

        ValidationUtil::validate($customer->getMiddleName(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::MIDDLE_NAME, AppErrorCodes::MIDDLE_NAME_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidMiddleName'));

        ValidationUtil::validate($customer->getGender(), DataType::ENUM, Mandatory::NO, $exception, HttpParamConstants::GENDER, AppErrorCodes::GENDER_INVALID, CommonUtil::getValidationMethod(Gender::class, 'isValidValue'));

        ValidationUtil::validate($customer->getDateOfbirth(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::DATE_OF_BIRTH, AppErrorCodes::DATE_OF_BIRTH_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDOB'));

        ValidationUtil::validate($customer->getSsn(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::SSN, AppErrorCodes::SSN_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidSSN'));

        ValidationUtil::validate($customer->getLocale(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::LOCALE, AppErrorCodes::LOCALE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidLocale'));

        ValidationUtil::validate($customer->getEmail(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::EMAIL, AppErrorCodes::EMAIL_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidEmail'));

        ValidationUtil::validate($customer->getHomePhone(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::HOME_PHONE, AppErrorCodes::HOME_PHONE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidHomePhone'));

        ValidationUtil::validate($customer->getWorkPhone(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::WORK_PHONE, AppErrorCodes::WORK_PHONE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidWorkPhone'));

        ValidationUtil::validate($customer->getMobilePhone(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::MOBILE_PHONE, AppErrorCodes::MOBILE_PHONE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidMobilePhone'));

        ValidationUtil::validate($customer->getAddress(), DataType::OBJECT, Mandatory::NO, $exception, HttpParamConstants::ADDRESS, '', CommonUtil::getValidationMethod(ValidationUtil::class, 'validateCustomerAddress'));

        ValidationUtil::validate($customer->getStatus(), DataType::ENUM, Mandatory::NO, $exception, HttpParamConstants::STATUS, AppErrorCodes::CUSTOMER_STATUS_INVALID, CommonUtil::getValidationMethod(CustomerStatus::class, 'isValidValue'));

        ValidationUtil::validate($customer->getCustomerAccounts(), DataType::OBJECT, Mandatory::NO, $exception, HttpParamConstants::CUSTOMER_ACCOUNTS, AppErrorCodes::SYSTEM_ERROR, CommonUtil::getValidationMethod(__CLASS__, 'validateCustomerAccounts'));

        ValidationUtil::validate($customer->getComments(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::COMMENTS, AppErrorCodes::COMMENTS_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidComments'));
    }

    public static function validateCustomerAccounts(CustomerAccounts $customerAccounts, OrbipayApiException $exception, $segment = '')
    {
        ValidationUtil::validate($customerAccounts->getList(), DataType::GROUP, Mandatory::NO, $exception, $segment, AppErrorCodes::SYSTEM_ERROR, CommonUtil::getValidationMethod(__CLASS__, 'validateCustomerAccountsList'));
    }

    public static function validateCustomerAccountsList(array $list, OrbipayApiException $exception, $baseSegment = '', $appErrorCode = '')
    {
        for ($i = 0; $i < count($list); $i++) {
            $segment = $baseSegment . HttpParamConstants::LEFT_SQUARE_BRACE . $i
                . HttpParamConstants::RIGHT_SQUARE_BRACE;
            ValidationUtil::validate($list[$i], DataType::OBJECT, Mandatory::YES, $exception, $segment, $appErrorCode, CommonUtil::getValidationMethod(__CLASS__, 'validateCustomerAccount'));
        }
    }

    public function validateCustomerAccount(CustomerAccount $customerAccount, OrbipayApiException $exception, $segment = '')
    {
        $segment .= HttpParamConstants::DOT;

        ValidationUtil::validate($customerAccount->getAccountHolderName(), DataType::STRING, Mandatory::YES, $exception, $segment . HttpParamConstants::ACCOUNT_HOLDER_NAME, AppErrorCodes::CUSTOMER_ACCOUNT_HOLDER_NAME_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAccountHolderName'));

        ValidationUtil::validate($customerAccount->getNickname(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::NICKNAME, AppErrorCodes::CUSTOMER_ACCOUNT_NICKNAME_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidNickname'));

        ValidationUtil::validate($customerAccount->getCustomerAccountReference(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::CUSTOMER_ACCOUNT_REFERENCE, AppErrorCodes::CUSTOMER_ACCOUNT_REFERENCE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountReference'));

        ValidationUtil::validate($customerAccount->getAccountNumber(), DataType::STRING, Mandatory::YES, $exception, $segment . HttpParamConstants::ACCOUNT_NUMBER, AppErrorCodes::CUSTOMER_ACCOUNT_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountNumber'));

        ValidationUtil::validate($customerAccount->getCurrentBalance(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::CURRENT_BALANCE, AppErrorCodes::CURRENT_BALANCE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountBalance'));

        ValidationUtil::validate($customerAccount->getCurrentStatementBalance(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::CURRENT_STATEMENT_BALANCE, AppErrorCodes::CURRENT_STATEMENT_BALANCE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountBalance'));

        ValidationUtil::validate($customerAccount->getMinimumPaymentDue(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::MINIMUM_PAYMENT_DUE, AppErrorCodes::MINIMUM_PAYMENT_DUE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountBalance'));

        ValidationUtil::validate($customerAccount->getPastAmountDue(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::PAST_AMOUNT_DUE, AppErrorCodes::PAST_AMOUNT_DUE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountBalance'));

        ValidationUtil::validate($customerAccount->getPaymentDueDate(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::PAYMENT_DUE_DATE, AppErrorCodes::PAYMENT_DUE_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));

        ValidationUtil::validate($customerAccount->getStatementDate(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::STATEMENT_DATE, AppErrorCodes::STATEMENT_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));

        ValidationUtil::validate($customerAccount->getAddress(), DataType::OBJECT, Mandatory::NO, $exception, $segment . HttpParamConstants::ADDRESS, '', CommonUtil::getValidationMethod(ValidationUtil::class, 'validateCustomerAccountAddress'));
        
        ValidationUtil::validate($customerAccount->getStatus(), DataType::ENUM, Mandatory::NO, $exception, HttpParamConstants::STATUS, AppErrorCodes::CUSTOMER_ACCOUNT_STATUS_INVALID, CommonUtil::getValidationMethod(CustomerAccountStatus::class, 'isValidValue'));
    }


    /**
     * @param null $apiResponse
     * @return Customer
     */
    private function wrapResponse($apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($customerResponse, $statusCode, $headers) = $apiResponse;

        $customer = ResponseWrapper::wrapCustomer($customerResponse);
        $customer->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $customer;
    }
}