<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\CustomerAccount;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccount;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\CustomerAccountApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class GetCustomerAccountHandler
{
    /**
     * @param CustomerAccount $customerAccount
     * @return CustomerAccount
     * @throws OrbipayApiException
     */
    public function process(CustomerAccount $customerAccount)
    {
        $this->validate($customerAccount);
        return $this->callAPI($customerAccount);
    }

    /**
     * @param CustomerAccount $customerAccount
     * @return CustomerAccount
     * @throws OrbipayApiException
     */
    private function callAPI(CustomerAccount $customerAccount)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($customerAccount->getApiEndPoint());
        $configuration->setClientKey($customerAccount->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($customerAccount->getInvocationContext()->getSecret());

        $customerAccountApi = new CustomerAccountApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\CustomerAccountResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $customerAccountApi->getCustomerAccount($customerAccount->getInvocationContext()->getChannel(),
                $customerAccount->getInvocationContext()->getClientKey(),
                $customerAccount->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $customerAccount->getInvocationContext()->getIdempotentRequestKey(),
                $customerAccount->getInvocationContext()->getRequestor(),
                $customerAccount->getInvocationContext()->getRequestorType()->value(),
                $customerAccount->getPathParamData()->getIdCustomerAccount(),
                $customerAccount->getPathParamData()->getIdCustomer(),
                RequestWrapper::wrapXOPAYHeaders($customerAccount->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customerAccount, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $customerAccount);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customerAccount, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    /**
     * @param CustomerAccount $customerAccount
     * @throws OrbipayApiException
     */
    private function validate(CustomerAccount $customerAccount = null)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $customerAccount);

        ValidationUtil::validate($customerAccount->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($customerAccount->getPathParamData()->getIdCustomerAccount(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER_ACCOUNT, AppErrorCodes::ID_CUSTOMER_ACCOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));

        ValidationUtil::validateInvocationContext($customerAccount->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    /**
     * @param array $apiResponse
     * @return CustomerAccount
     */
    private function wrapResponse(array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($customerAccountResponse, $statusCode, $headers) = $apiResponse;

        $customerAccount = ResponseWrapper::wrapCustomerAccount($customerAccountResponse);
        $customerAccount->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $customerAccount;
    }
}