<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\CustomerAccount;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\CustomerAccountStatus;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccounts;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\CustomerAccountApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class RetrieveCustomerAccountsHandler
{
    /**
     * @param CustomerAccounts $customerAccounts
     * @return CustomerAccounts
     * @throws OrbipayApiException
     */
    public function process(CustomerAccounts $customerAccounts)
    {
        $this->validate($customerAccounts);
        return $this->callAPI($customerAccounts);
    }

    /**
     * @param CustomerAccounts $customerAccounts
     * @return CustomerAccounts
     * @throws OrbipayApiException
     */
    private function callAPI(CustomerAccounts $customerAccounts)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($customerAccounts->getApiEndPoint());
        $configuration->setClientKey($customerAccounts->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($customerAccounts->getInvocationContext()->getSecret());

        $customerAccountApi = new CustomerAccountApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\CustomerAccountsResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {

            if (ValidationUtil::isNonEmptyString($customerAccounts->getQueryParamData()->getQueryId())) {

                $response = $customerAccountApi->retrieveCustomerAccountsPage($customerAccounts->getInvocationContext()->getChannel(),
                    $customerAccounts->getInvocationContext()->getClientKey(),
                    $customerAccounts->getInvocationContext()->getProduct()->value(),
                    TimeUtil::getTimeStamp(),
                    $customerAccounts->getInvocationContext()->getIdempotentRequestKey(),
                    $customerAccounts->getInvocationContext()->getRequestor(),
                    $customerAccounts->getInvocationContext()->getRequestorType()->value(),
                    $customerAccounts->getPathParamData()->getIdCustomer(),
                    $customerAccounts->getQueryParamData()->getQueryId(),
                    RequestWrapper::wrapXOPAYHeaders($customerAccounts->getInvocationContext()->getAttributes()),
                    $customerAccounts->getQueryParamData()->getPageSize(),
                    $customerAccounts->getQueryParamData()->getToIndex(),
                    $customerAccounts->getQueryParamData()->getFromIndex());

            } else {

                $form_params = [];
                $form_params[HttpParamConstants::ACCOUNT_NUMBER] = $customerAccounts->getFormParamData()->getAccountNumber();
                $form_params[HttpParamConstants::STATUS] = $customerAccounts->getFormParamData()->getStatus();

                $response = $customerAccountApi->retrieveCustomerAccounts($customerAccounts->getInvocationContext()->getChannel(),
                    $customerAccounts->getInvocationContext()->getClientKey(),
                    $customerAccounts->getInvocationContext()->getProduct()->value(),
                    TimeUtil::getTimeStamp(),
                    $customerAccounts->getInvocationContext()->getIdempotentRequestKey(),
                    $customerAccounts->getInvocationContext()->getRequestor(),
                    $customerAccounts->getInvocationContext()->getRequestorType()->value(),
                    $customerAccounts->getPathParamData()->getIdCustomer(),
                    RequestWrapper::wrapXOPAYHeaders($customerAccounts->getInvocationContext()->getAttributes()),
                    $customerAccounts->getQueryParamData()->getPageSize(), $form_params);
            }
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customerAccounts, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $customerAccounts);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customerAccounts, $e->getCode(), $e);
        }

        return $this->wrapResponse($customerAccounts, $response);
    }

    /**
     * @param CustomerAccounts $customerAccounts
     * @throws OrbipayApiException
     */
    private function validate(CustomerAccounts $customerAccounts = null)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $customerAccounts);

        ValidationUtil::validate($customerAccounts->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($customerAccounts->getFormParamData()->getAccountNumber(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::ACCOUNT_NUMBER, AppErrorCodes::FORM_PARAM_CUSTOMER_ACCOUNT_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCustomerAccountNumber'));
        ValidationUtil::validate($customerAccounts->getFormParamData()->getStatus(), DataType::GROUP, Mandatory::NO, $exception, HttpParamConstants::STATUS, AppErrorCodes::CUSTOMER_ACCOUNT_STATUS_INVALID, CommonUtil::getValidationMethod(CustomerAccountStatus::class, 'areValidValues'));

        ValidationUtil::validate($customerAccounts->getQueryParamData()->getPageSize(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::PAGE_SIZE, AppErrorCodes::PAGE_SIZE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidPageSize'));

        ValidationUtil::validate($customerAccounts->getQueryParamData()->getQueryId(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_ID, AppErrorCodes::QUERY_ID_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidQueryId'));
        ValidationUtil::validate($customerAccounts->getQueryParamData()->getToIndex(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::TO_INDEX, AppErrorCodes::TO_INDEX_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidToIndex'));
        ValidationUtil::validate($customerAccounts->getQueryParamData()->getFromIndex(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::FROM_INDEX, AppErrorCodes::FROM_INDEX_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidFromIndex'));


        ValidationUtil::validateInvocationContext($customerAccounts->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    /**
     * @param CustomerAccounts $customerAccounts
     * @param array $apiResponse
     * @return CustomerAccounts
     */
    private function wrapResponse(CustomerAccounts $customerAccounts, array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($customerAccountResponse, $statusCode, $headers) = $apiResponse;

        ResponseWrapper::wrapCustomerAccounts($customerAccounts, $customerAccountResponse);
        $customerAccounts->withContext(ResponseWrapper::getInvocationContext($headers, $customerAccounts), $customerAccounts->getInvocationMode(), $customerAccounts->getApiEndPoint());
        return $customerAccounts;
    }
}