<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\FundingAccount;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\FundingAccount\FundingAccount;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\FundingAccountApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;
use Orbipay\PaymentsApi\Client\Swagger\Model\DeleteFundingAccountRequest;

class DeleteFundingAccountHandler
{
    /**
     * @param FundingAccount $fundingAccount
     * @return FundingAccount
     * @throws OrbipayApiException
     */
    public function process(FundingAccount $fundingAccount)
    {
        $this->validate($fundingAccount);
        return $this->callAPI($fundingAccount);
    }

    /**
     * @param FundingAccount $fundingAccount
     * @return FundingAccount
     * @throws OrbipayApiException
     */
    private function callAPI(FundingAccount $fundingAccount)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($fundingAccount->getApiEndPoint());
        $configuration->setClientKey($fundingAccount->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($fundingAccount->getInvocationContext()->getSecret());

        $fundingAccountApi = new FundingAccountApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\DeleteResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $fundingAccountApi->deleteFundingAccount($fundingAccount->getInvocationContext()->getChannel(),
                $fundingAccount->getInvocationContext()->getClientKey(),
                $fundingAccount->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $fundingAccount->getInvocationContext()->getIdempotentRequestKey(),
                $fundingAccount->getInvocationContext()->getRequestor(),
                $fundingAccount->getInvocationContext()->getRequestorType()->value(),
                $fundingAccount->getPathParamData()->getIdFundingAccount(),
                $fundingAccount->getPathParamData()->getIdCustomer(),
                $this->getBody($fundingAccount),
                RequestWrapper::wrapXOPAYHeaders($fundingAccount->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $fundingAccount, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $fundingAccount);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $fundingAccount, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    /**
     * @param FundingAccount $fundingAccount
     * @throws OrbipayApiException
     */
    private function validate(FundingAccount $fundingAccount)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $fundingAccount);

        ValidationUtil::validate($fundingAccount->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));

        ValidationUtil::validate($fundingAccount->getPathParamData()->getIdFundingAccount(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_FUNDING_ACCOUNT, AppErrorCodes::EXT_ISSUED_FUNDING_ACCOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));

        ValidationUtil::validate($fundingAccount->getComments(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::COMMENTS, AppErrorCodes::COMMENTS_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidComments'));

        ValidationUtil::validateInvocationContext($fundingAccount->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    private function getBody(FundingAccount $fundingAccount)
    {
        $body = new DeleteFundingAccountRequest();
        $body->setComments($fundingAccount->getComments());
        return $body;
    }

    /**
     * @param array $apiResponse
     * @return FundingAccount
     */
    private function wrapResponse(array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($fundingAccountResponse, $statusCode, $headers) = $apiResponse;

        $fundingAccount = ResponseWrapper::wrapDeleteFundingAccountResponse($fundingAccountResponse);
        $fundingAccount->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $fundingAccount;
    }
}