<?php

namespace Orbipay\PaymentsApi\Client\Handler\FundingAccount;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\FundingAccountHolderType;
use Orbipay\PaymentsApi\Client\Enum\FundingAccountSubtype;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\FundingAccount\FundingAccount;
use Orbipay\PaymentsApi\Client\Swagger\Model\ReplaceFundingAccountRequest;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\FundingAccountApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;


class ReplaceFundingAccountHandler
{
    /**
     * @param FundingAccount $fundingAccount
     * @return FundingAccount
     * @throws OrbipayApiException
     */
    public function process(FundingAccount $fundingAccount)
    {
        $this->validate($fundingAccount);
        return $this->callAPI($fundingAccount);
    }

    /**
     * @param FundingAccount $fundingAccount
     * @return FundingAccount
     * @throws OrbipayApiException
     */
    private function callAPI(FundingAccount $fundingAccount)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($fundingAccount->getApiEndPoint());
        $configuration->setClientKey($fundingAccount->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($fundingAccount->getInvocationContext()->getSecret());

        $fundingAccountApi = new FundingAccountApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\FundingAccountResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $fundingAccountApi->replaceFundingAccount($fundingAccount->getInvocationContext()->getChannel(),
                $fundingAccount->getInvocationContext()->getClientKey(),
                $fundingAccount->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $fundingAccount->getInvocationContext()->getIdempotentRequestKey(),
                $fundingAccount->getInvocationContext()->getRequestor(),
                $fundingAccount->getInvocationContext()->getRequestorType()->value(),
                $fundingAccount->getPathParamData()->getIdFundingAccount(),
                $fundingAccount->getPathParamData()->getIdCustomer(),
                $this->getBody($fundingAccount),
                RequestWrapper::wrapXOPAYHeaders($fundingAccount->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $fundingAccount, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $fundingAccount);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $fundingAccount, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    /**
     * @param FundingAccount $fundingAccount
     * @throws OrbipayApiException
     */
    private function validate(FundingAccount $fundingAccount)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $fundingAccount);

        ValidationUtil::validate($fundingAccount->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($fundingAccount->getPathParamData()->getIdFundingAccount(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_FUNDING_ACCOUNT, AppErrorCodes::EXT_ISSUED_FUNDING_ACCOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));

        $this->validateFundingAccount($fundingAccount, $exception);

        ValidationUtil::validateInvocationContext($fundingAccount->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    private function validateFundingAccount(FundingAccount $fundingAccount, OrbipayApiException $exception)
    {
        ValidationUtil::validate($fundingAccount->getAccountHolderName(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::ACCOUNT_HOLDER_NAME, AppErrorCodes::FUNDING_ACCOUNT_HOLDER_NAME_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAccountHolderName'));

        ValidationUtil::validate($fundingAccount->getNickname(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::NICKNAME, AppErrorCodes::FUNDING_ACCOUNT_NICKNAME_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidNickname'));

        ValidationUtil::validate($fundingAccount->getAccountNumber(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::ACCOUNT_NUMBER, AppErrorCodes::FUNDING_ACCOUNT_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidFundingAccountNumber'));

        ValidationUtil::validate($fundingAccount->getAbaRoutingNumber(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::ABA_ROUTING_NUMBER, AppErrorCodes::ABA_ROUTING_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidABARoutingNumber'));

        ValidationUtil::validate($fundingAccount->getExpiryDate(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::EXPIRY_DATE, AppErrorCodes::EXPIRY_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidExpiryDate'));

        ValidationUtil::validate($fundingAccount->getAccountHolderType(), DataType::ENUM, Mandatory::NO, $exception, HttpParamConstants::ACCOUNT_HOLDER_TYPE, AppErrorCodes::ACCOUNT_HOLDER_TYPE_INVALID, CommonUtil::getValidationMethod(FundingAccountHolderType::class, 'isValidValue'));

        ValidationUtil::validate($fundingAccount->getAddress(), DataType::OBJECT, Mandatory::NO, $exception, HttpParamConstants::ADDRESS, '', CommonUtil::getValidationMethod(ValidationUtil::class, 'validateFundingAccountAddress'));

        ValidationUtil::validate($fundingAccount->getAccountSubtype(), DataType::ENUM, Mandatory::NO, $exception, HttpParamConstants::ACCOUNT_SUBTYPE, AppErrorCodes::FUNDING_ACCOUNT_SUBTYPE_INVALID, CommonUtil::getValidationMethod(FundingAccountSubtype::class, 'isValidValue'));

        ValidationUtil::validate($fundingAccount->getCardCvvNumber(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::CARD_CVV_NUMBER, AppErrorCodes::CARD_CVV_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCardCVVNumber'));

        ValidationUtil::validate($fundingAccount->getComments(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::COMMENTS, AppErrorCodes::COMMENTS_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidComments'));
    }

    /**
     * @param array $apiResponse
     * @return FundingAccount
     */
    private function wrapResponse(array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($fundingAccountResponse, $statusCode, $headers) = $apiResponse;

        $fundingAccount = ResponseWrapper::wrapFundingAccount($fundingAccountResponse);
        $fundingAccount->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $fundingAccount;
    }

    private function getBody(FundingAccount $fundingAccount)
    {
        $body = new ReplaceFundingAccountRequest();
        $body->setAccountHolderName($fundingAccount->getAccountHolderName());
        $body->setNickname($fundingAccount->getNickname());
        $body->setAddress(RequestWrapper::wrapAddress($fundingAccount->getAddress()));
        $body->setAccountNumber($fundingAccount->getAccountNumber());
        $body->setAbaRoutingNumber($fundingAccount->getAbaRoutingNumber());
        $body->setExpiryDate($fundingAccount->getExpiryDate());
        $body->setAccountHolderType($fundingAccount->getAccountHolderType() !== null ? $fundingAccount->getAccountHolderType()->value() : null);
        $body->setCustomFields($fundingAccount->getCustomFields());
        $body->setAccountSubtype($fundingAccount->getAccountSubtype() !== null ? $fundingAccount->getAccountSubtype()->value() : null);
        $body->setCardCvvNumber($fundingAccount->getCardCvvNumber());
        $body->setComments($fundingAccount->getComments());
        return $body;
    }
}