<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\FundingAccount;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\FundingAccountStatus;
use Orbipay\PaymentsApi\Client\Enum\FundingAccountSubtype;
use Orbipay\PaymentsApi\Client\Enum\FundingAccountType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\FundingAccount\FundingAccounts;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\FundingAccountApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class RetrieveFundingAccountsHandler
{
    /**
     * @param FundingAccounts $fundingAccounts
     * @return FundingAccounts
     * @throws OrbipayApiException
     */
    public function process(FundingAccounts $fundingAccounts)
    {
        $this->validate($fundingAccounts);
        return $this->callAPI($fundingAccounts);
    }

    /**
     * @param FundingAccounts $fundingAccounts
     * @return FundingAccounts
     * @throws OrbipayApiException
     */
    private function callAPI(FundingAccounts $fundingAccounts)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($fundingAccounts->getApiEndPoint());
        $configuration->setClientKey($fundingAccounts->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($fundingAccounts->getInvocationContext()->getSecret());

        $fundingAccountApi = new FundingAccountApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\FundingAccountsResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {


            if (ValidationUtil::isNonEmptyString($fundingAccounts->getQueryParamData()->getQueryId())) {

                $response = $fundingAccountApi->retrieveFundingAccountsPage($fundingAccounts->getInvocationContext()->getChannel(),
                    $fundingAccounts->getInvocationContext()->getClientKey(),
                    $fundingAccounts->getInvocationContext()->getProduct()->value(),
                    TimeUtil::getTimeStamp(),
                    $fundingAccounts->getInvocationContext()->getIdempotentRequestKey(),
                    $fundingAccounts->getInvocationContext()->getRequestor(),
                    $fundingAccounts->getInvocationContext()->getRequestorType()->value(),
                    $fundingAccounts->getPathParamData()->getIdCustomer(),
                    $fundingAccounts->getQueryParamData()->getQueryId(),
                    RequestWrapper::wrapXOPAYHeaders($fundingAccounts->getInvocationContext()->getAttributes()),
                    $fundingAccounts->getQueryParamData()->getPageSize(),
                    $fundingAccounts->getQueryParamData()->getToIndex(),
                    $fundingAccounts->getQueryParamData()->getFromIndex());

            } else {

                $form_params = [];
                $form_params[HttpParamConstants::ACCOUNT_NUMBER] = $fundingAccounts->getFormParamData()->getAccountNumber();
                $form_params[HttpParamConstants::STATUS] = $fundingAccounts->getFormParamData()->getStatus();
                $form_params[HttpParamConstants::ABA_ROUTING_NUMBER] = $fundingAccounts->getFormParamData()->getAbaRoutingNumber();
                $form_params[HttpParamConstants::ACCOUNT_TYPE] = $fundingAccounts->getFormParamData()->getAccountType();
                $form_params[HttpParamConstants::ACCOUNT_SUBTYPE] = $fundingAccounts->getFormParamData()->getAccountSubtype();

                $response = $fundingAccountApi->retrieveFundingAccounts($fundingAccounts->getInvocationContext()->getChannel(),
                    $fundingAccounts->getInvocationContext()->getClientKey(),
                    $fundingAccounts->getInvocationContext()->getProduct()->value(),
                    TimeUtil::getTimeStamp(),
                    $fundingAccounts->getInvocationContext()->getIdempotentRequestKey(),
                    $fundingAccounts->getInvocationContext()->getRequestor(),
                    $fundingAccounts->getInvocationContext()->getRequestorType()->value(),
                    $fundingAccounts->getPathParamData()->getIdCustomer(),
                    RequestWrapper::wrapXOPAYHeaders($fundingAccounts->getInvocationContext()->getAttributes()),
                    $fundingAccounts->getQueryParamData()->getPageSize(), $form_params);

            }
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $fundingAccounts, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $fundingAccounts);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $fundingAccounts, $e->getCode(), $e);
        }

        return $this->wrapResponse($fundingAccounts, $response);
    }

    /**
     * @param FundingAccounts $fundingAccounts
     * @throws OrbipayApiException
     */
    private function validate(FundingAccounts $fundingAccounts)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $fundingAccounts);

        ValidationUtil::validate($fundingAccounts->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($fundingAccounts->getFormParamData()->getAccountNumber(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::ACCOUNT_NUMBER, AppErrorCodes::FORM_PARAM_FUNDING_ACCOUNT_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidFundingAccountNumber'));
        ValidationUtil::validate($fundingAccounts->getFormParamData()->getAbaRoutingNumber(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::ABA_ROUTING_NUMBER, AppErrorCodes::FORM_PARAM_FUNDING_ACCOUNT_ABA_ROUTING_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidABARoutingNumber'));

        ValidationUtil::validate($fundingAccounts->getFormParamData()->getStatus(), DataType::GROUP, Mandatory::NO, $exception, HttpParamConstants::STATUS, AppErrorCodes::FORM_PARAM_FUNDING_ACCOUNT_STATUS_INVALID, CommonUtil::getValidationMethod(FundingAccountStatus::class, 'areValidValues'));
        ValidationUtil::validate($fundingAccounts->getFormParamData()->getAccountType(), DataType::GROUP, Mandatory::NO, $exception, HttpParamConstants::ACCOUNT_TYPE, AppErrorCodes::FORM_PARAM_FUNDING_ACCOUNT_TYPE_INVALID, CommonUtil::getValidationMethod(FundingAccountType::class, 'areValidValues'));
        ValidationUtil::validate($fundingAccounts->getFormParamData()->getAccountSubtype(), DataType::GROUP, Mandatory::NO, $exception, HttpParamConstants::ACCOUNT_SUBTYPE, AppErrorCodes::FORM_PARAM_FUNDING_ACCOUNT_SUBTYPE_INVALID, CommonUtil::getValidationMethod(FundingAccountSubtype::class, 'areValidValues'));

        ValidationUtil::validate($fundingAccounts->getQueryParamData()->getPageSize(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::PAGE_SIZE, AppErrorCodes::PAGE_SIZE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidPageSize'));

        ValidationUtil::validate($fundingAccounts->getQueryParamData()->getQueryId(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_ID, AppErrorCodes::QUERY_ID_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidQueryId'));
        ValidationUtil::validate($fundingAccounts->getQueryParamData()->getToIndex(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::TO_INDEX, AppErrorCodes::TO_INDEX_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidToIndex'));
        ValidationUtil::validate($fundingAccounts->getQueryParamData()->getFromIndex(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::FROM_INDEX, AppErrorCodes::FROM_INDEX_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidFromIndex'));


        ValidationUtil::validateInvocationContext($fundingAccounts->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    /**
     * @param FundingAccounts $fundingAccounts
     * @param array $apiResponse
     * @return FundingAccounts
     */
    private function wrapResponse(FundingAccounts $fundingAccounts, array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($fundingAccountResponse, $statusCode, $headers) = $apiResponse;

        ResponseWrapper::wrapFundingAccounts($fundingAccounts, $fundingAccountResponse);
        $fundingAccounts->withContext(ResponseWrapper::getInvocationContext($headers, $fundingAccounts), $fundingAccounts->getInvocationMode(), $fundingAccounts->getApiEndPoint());
        return $fundingAccounts;
    }
}