<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\Payment;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\Payment\Payment;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\PaymentApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;
use Orbipay\PaymentsApi\Client\Swagger\Model\CreatePaymentRequest;

class CreatePaymentHandler
{
    /**
     * @param Payment $payment
     * @return Payment
     * @throws OrbipayApiException
     */
    public function process(Payment $payment)
    {
        $this->validate($payment);
        return $this->callAPI($payment);
    }

    /**
     * @param Payment $payment
     * @return Payment
     * @throws OrbipayApiException
     */
    private function callAPI(Payment $payment)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($payment->getApiEndPoint());
        $configuration->setClientKey($payment->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($payment->getInvocationContext()->getSecret());

        $paymentApi = new PaymentApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\PaymentResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $paymentApi->createPayment($payment->getInvocationContext()->getChannel(),
                $payment->getInvocationContext()->getClientKey(),
                $payment->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $payment->getInvocationContext()->getIdempotentRequestKey(),
                $payment->getInvocationContext()->getRequestor(),
                $payment->getInvocationContext()->getRequestorType()->value(),
                $payment->getPathParamData()->getIdCustomer(),
                $this->getBody($payment),
                RequestWrapper::wrapXOPAYHeaders($payment->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $payment, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $payment);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $payment, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    /**
     * @param Payment $payment
     * @throws OrbipayApiException
     */
    private function validate(Payment $payment)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $payment);

        ValidationUtil::validate($payment->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));

        $this->validatePayment($payment, $exception);

        ValidationUtil::validateInvocationContext($payment->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    private function validatePayment(Payment $payment, OrbipayApiException $exception)
    {

        ValidationUtil::validate($payment->getAmount(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::AMOUNT, AppErrorCodes::AMOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAmount'));

        ValidationUtil::validate($payment->getCardCvvNumber(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::CARD_CVV_NUMBER, AppErrorCodes::CARD_CVV_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCardCVVNumber'));

        ValidationUtil::validate($payment->getPaymentDate(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::PAYMENT_DATE, AppErrorCodes::PAYMENT_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));

        ValidationUtil::validate($payment->getPaymentRequestDate(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::PAYMENT_REQUEST_DATE, AppErrorCodes::PAYMENT_REQUEST_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));

        ValidationUtil::validate($payment->getFee(), DataType::OBJECT, Mandatory::NO, $exception, HttpParamConstants::FEE, '', CommonUtil::getValidationMethod(ValidationUtil::class, 'validateFee'));

        ValidationUtil::validate($payment->getCustomer(), DataType::OBJECT, Mandatory::YES, $exception, HttpParamConstants::CUSTOMER, AppErrorCodes::ERROR_SEGMENT_MISSING, CommonUtil::getValidationMethod(ValidationUtil::class, 'validateCustomer'));

        ValidationUtil::validate($payment->getCustomerAccount(), DataType::OBJECT, Mandatory::YES, $exception, HttpParamConstants::CUSTOMER_ACCOUNT, AppErrorCodes::ERROR_SEGMENT_MISSING, CommonUtil::getValidationMethod(ValidationUtil::class, 'validateCustomerAccount'));

        ValidationUtil::validate($payment->getFundingAccount(), DataType::OBJECT, Mandatory::YES, $exception, HttpParamConstants::FUNDING_ACCOUNT, AppErrorCodes::ERROR_SEGMENT_MISSING, CommonUtil::getValidationMethod(ValidationUtil::class, 'validateFundingAccount'));

        ValidationUtil::validate($payment->getPaymentReference(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::PAYMENT_REFERENCE, AppErrorCodes::PAYMENT_REFERENCE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidPaymentReference'));

        ValidationUtil::validate($payment->getComments(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::COMMENTS, AppErrorCodes::COMMENTS_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidComments'));

    }

    /**
     * @param array $apiResponse
     * @return Payment
     */
    private function wrapResponse(array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($paymentResponse, $statusCode, $headers) = $apiResponse;

        $payment = ResponseWrapper::wrapPayment($paymentResponse);
        $payment->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $payment;
    }

    private function getBody(Payment $payment)
    {
        $body = new CreatePaymentRequest();
        $body->setAmount($payment->getAmount());
        $body->setCardCvvNumber($payment->getCardCvvNumber());
        $body->setPaymentDate($payment->getPaymentDate());
        $body->setPaymentRequestDate($payment->getPaymentRequestDate());
        $body->setPaymentAmountType($payment->getPaymentAmountType() !== null ? $payment->getPaymentAmountType()->value() : null);
        $body->setFee(RequestWrapper::getFeeRequest($payment->getFee()));
        $body->setCustomer(RequestWrapper::getCustomerMinimalRequest($payment->getCustomer()));
        $body->setCustomerAccount(RequestWrapper::getCustomerAccountMinimalRequest($payment->getCustomerAccount()));
        $body->setFundingAccount(RequestWrapper::getFundingAccountMinimalRequest($payment->getFundingAccount()));
        $body->setPaymentReference($payment->getPaymentReference());
        $body->setComments($payment->getComments());
        $body->setCustomFields($payment->getCustomFields());
        return $body;
    }
}