<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\Payment;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\Payment\Payment;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\PaymentApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class GetPaymentHandler
{
    /**
     * @param Payment $payment
     * @return Payment
     * @throws OrbipayApiException
     */
    public function process(Payment $payment)
    {
        $this->validate($payment);
        return $this->callAPI($payment);
    }

    /**
     * @param Payment $payment
     * @return Payment
     * @throws OrbipayApiException
     */
    private function callAPI(Payment $payment)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($payment->getApiEndPoint());
        $configuration->setClientKey($payment->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($payment->getInvocationContext()->getSecret());

        $paymentApi = new PaymentApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\PaymentResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $paymentApi->getPayment($payment->getInvocationContext()->getChannel(),
                $payment->getInvocationContext()->getClientKey(),
                $payment->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $payment->getInvocationContext()->getIdempotentRequestKey(),
                $payment->getInvocationContext()->getRequestor(),
                $payment->getInvocationContext()->getRequestorType()->value(),
                $payment->getPathParamData()->getIdPayment(),
                $payment->getPathParamData()->getIdCustomer(),
                RequestWrapper::wrapXOPAYHeaders($payment->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $payment, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $payment);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $payment, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    /**
     * @param Payment $payment
     * @throws OrbipayApiException
     */
    private function validate(Payment $payment)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $payment);

        ValidationUtil::validate($payment->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($payment->getPathParamData()->getIdPayment(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_PAYMENT, AppErrorCodes::ID_PAYMENT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));

        ValidationUtil::validateInvocationContext($payment->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    /**
     * @param array $apiResponse
     * @return Payment
     */
    private function wrapResponse(array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($paymentResponse, $statusCode, $headers) = $apiResponse;

        $payment = ResponseWrapper::wrapPayment($paymentResponse);
        $payment->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $payment;
    }
}