<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\Payment;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Enum\PaymentScheduleType;
use Orbipay\PaymentsApi\Client\Enum\PaymentStatus;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\Payment\Payments;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\PaymentApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class RetrievePaymentsHandler
{
    /**
     * @param Payments $payments
     * @return Payments
     * @throws OrbipayApiException
     */
    public function process(Payments $payments)
    {
        $this->validate($payments);
        return $this->callAPI($payments);
    }

    /**
     * @param Payments $payments
     * @return Payments
     * @throws OrbipayApiException
     */
    private function callAPI(Payments $payments)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($payments->getApiEndPoint());
        $configuration->setClientKey($payments->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($payments->getInvocationContext()->getSecret());

        $paymentApi = new PaymentApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\PaymentsResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $paymentApi->retrievePayments($payments->getInvocationContext()->getChannel(),
                $payments->getInvocationContext()->getClientKey(),
                $payments->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $payments->getInvocationContext()->getIdempotentRequestKey(),
                $payments->getInvocationContext()->getRequestor(),
                $payments->getInvocationContext()->getRequestorType()->value(),
                RequestWrapper::wrapXOPAYHeaders($payments->getInvocationContext()->getAttributes()),
                $payments->getQueryParamData()->getIdCustomer(),
                $payments->getQueryParamData()->getIdFundingAccount(),
                $payments->getQueryParamData()->getIdCustomerAccount(),
                $payments->getQueryParamData()->getConfirmationNumber(),
                $payments->getQueryParamData()->getStatus(),
                $payments->getQueryParamData()->getPaymentScheduleType(),
                $payments->getQueryParamData()->getTxnPymntSchldType(),
                $payments->getQueryParamData()->getFromDate(),
                $payments->getQueryParamData()->getToDate(),
                $payments->getQueryParamData()->getPageSize(),
                $payments->getQueryParamData()->getQueryId(),
                $payments->getQueryParamData()->getToIndex(),
                $payments->getQueryParamData()->getFromIndex());
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $payments, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $payments);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $payments, $e->getCode(), $e);
        }

        return $this->wrapResponse($payments, $response);
    }

    /**
     * @param Payments $payments
     * @throws OrbipayApiException
     */
    private function validate(Payments $payments)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $payments);

        ValidationUtil::validate($payments->getQueryParamData()->getIdCustomer(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($payments->getQueryParamData()->getIdFundingAccount(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_PARAM_ID_FUNDING_ACCOUNT, AppErrorCodes::ID_FUNDING_ACCOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($payments->getQueryParamData()->getIdCustomerAccount(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_PARAM_ID_CUSTOMER_ACCOUNT, AppErrorCodes::ID_CUSTOMER_ACCOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($payments->getQueryParamData()->getConfirmationNumber(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::CONFIRMATION_NUMBER, AppErrorCodes::PAYMENT_CONFIRMATION_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidConfirmationNumber'));
        ValidationUtil::validate($payments->getQueryParamData()->getStatus(), DataType::GROUP, Mandatory::NO, $exception, HttpParamConstants::STATUS, AppErrorCodes::QUERY_PARAM_PAYMENT_STATUS_INVALID, CommonUtil::getValidationMethod(PaymentStatus::class, 'areValidValues'));
        ValidationUtil::validate($payments->getQueryParamData()->getPaymentScheduleType(), DataType::GROUP, Mandatory::NO, $exception, HttpParamConstants::PAYMENT_SCHEDULE_TYPE, AppErrorCodes::PAYMENT_SCHEDULE_TYPE_INVALID, CommonUtil::getValidationMethod(PaymentScheduleType::class, 'areValidValues'));
        ValidationUtil::validate($payments->getQueryParamData()->getFromDate(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::FROM_DATE, AppErrorCodes::PAYMENT_FROM_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));
        ValidationUtil::validate($payments->getQueryParamData()->getToDate(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::TO_DATE, AppErrorCodes::PAYMENT_TO_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));

        ValidationUtil::validate($payments->getQueryParamData()->getPageSize(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::PAGE_SIZE, AppErrorCodes::PAGE_SIZE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidPageSize'));

        ValidationUtil::validate($payments->getQueryParamData()->getQueryId(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_ID, AppErrorCodes::QUERY_ID_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidQueryId'));
        ValidationUtil::validate($payments->getQueryParamData()->getToIndex(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::TO_INDEX, AppErrorCodes::TO_INDEX_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidToIndex'));
        ValidationUtil::validate($payments->getQueryParamData()->getFromIndex(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::FROM_INDEX, AppErrorCodes::FROM_INDEX_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidFromIndex'));


        ValidationUtil::validateInvocationContext($payments->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    /**
     * @param Payments $payments
     * @param array $apiResponse
     * @return Payments
     */
    private function wrapResponse(Payments $payments, array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($paymentResponse, $statusCode, $headers) = $apiResponse;

        ResponseWrapper::wrapPayments($payments, $paymentResponse);
        $payments->withContext(ResponseWrapper::getInvocationContext($headers, $payments), $payments->getInvocationMode(), $payments->getApiEndPoint());
        return $payments;
    }
}