<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\PaymentSetup;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Enum\PaymentAmountType;
use Orbipay\PaymentsApi\Client\Enum\PaymentRecurringType;
use Orbipay\PaymentsApi\Client\Enum\PaymentSetupScheduleType;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\PaymentSetup\PaymentSchedule;
use Orbipay\PaymentsApi\Client\Model\PaymentSetup\PaymentSetup;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\PaymentSetupApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;
use Orbipay\PaymentsApi\Client\Swagger\Model\CreatePaymentSetupRequest;
use Orbipay\PaymentsApi\Client\Swagger\Model\PaymentScheduleVO;

class CreatePaymentSetupHandler
{
    /**
     * @param PaymentSetup $paymentSetup
     * @return PaymentSetup
     * @throws OrbipayApiException
     */
    public function process(PaymentSetup $paymentSetup)
    {
        $this->validate($paymentSetup);
        return $this->callAPI($paymentSetup);
    }

    /**
     * @param PaymentSetup $paymentSetup
     * @return PaymentSetup
     * @throws OrbipayApiException
     */
    private function callAPI(PaymentSetup $paymentSetup)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($paymentSetup->getApiEndPoint());
        $configuration->setClientKey($paymentSetup->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($paymentSetup->getInvocationContext()->getSecret());

        $paymentSetupApi = new PaymentSetupApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\PaymentSetupResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $paymentSetupApi->createPaymentSetup($paymentSetup->getInvocationContext()->getChannel(),
                $paymentSetup->getInvocationContext()->getClientKey(),
                $paymentSetup->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $paymentSetup->getInvocationContext()->getIdempotentRequestKey(),
                $paymentSetup->getInvocationContext()->getRequestor(),
                $paymentSetup->getInvocationContext()->getRequestorType()->value(),
                $paymentSetup->getPathParamData()->getIdCustomer(),
                $this->getBody($paymentSetup),
                RequestWrapper::wrapXOPAYHeaders($paymentSetup->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $paymentSetup, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $paymentSetup);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $paymentSetup, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    /**
     * @param PaymentSetup $paymentSetup
     * @throws OrbipayApiException
     */
    private function validate(PaymentSetup $paymentSetup)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $paymentSetup);

        ValidationUtil::validate($paymentSetup->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));

        $this->validatePaymentSetup($paymentSetup, $exception);

        ValidationUtil::validateInvocationContext($paymentSetup->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    private function validatePaymentSetup(PaymentSetup $paymentSetup, OrbipayApiException $exception)
    {
        ValidationUtil::validate($paymentSetup->getAmount(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::AMOUNT, AppErrorCodes::AMOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAmount'));

        ValidationUtil::validate($paymentSetup->getCardCvvNumber(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::CARD_CVV_NUMBER, AppErrorCodes::CARD_CVV_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCardCVVNumber'));

        ValidationUtil::validate($paymentSetup->getFee(), DataType::OBJECT, Mandatory::NO, $exception, HttpParamConstants::FEE, '', CommonUtil::getValidationMethod(ValidationUtil::class, 'validateFee'));

        ValidationUtil::validate($paymentSetup->getCustomer(), DataType::OBJECT, Mandatory::NO, $exception, HttpParamConstants::CUSTOMER, AppErrorCodes::ERROR_SEGMENT_MISSING, CommonUtil::getValidationMethod(ValidationUtil::class, 'validateCustomer'));

        ValidationUtil::validate($paymentSetup->getCustomerAccount(), DataType::OBJECT, Mandatory::NO, $exception, HttpParamConstants::CUSTOMER_ACCOUNT, AppErrorCodes::ERROR_SEGMENT_MISSING, CommonUtil::getValidationMethod(ValidationUtil::class, 'validateCustomerAccount'));

        ValidationUtil::validate($paymentSetup->getFundingAccount(), DataType::OBJECT, Mandatory::NO, $exception, HttpParamConstants::FUNDING_ACCOUNT, AppErrorCodes::ERROR_SEGMENT_MISSING, CommonUtil::getValidationMethod(ValidationUtil::class, 'validateFundingAccount'));

        ValidationUtil::validate($paymentSetup->getPaymentSetupReference(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::PAYMENT_SETUP_REFERENCE, AppErrorCodes::PAYMENT_SETUP_REFERENCE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidPaymentReference'));

        ValidationUtil::validate($paymentSetup->getComments(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::COMMENTS, AppErrorCodes::COMMENTS_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidComments'));

        ValidationUtil::validate($paymentSetup->getPaymentSetupScheduleType(), DataType::ENUM, Mandatory::YES, $exception, HttpParamConstants::PAYMENT_SETUP_SCHEDULE_TYPE, AppErrorCodes::PAYMENT_SETUP_SCHEDULE_TYPE_INVALID, CommonUtil::getValidationMethod(PaymentSetupScheduleType::class, 'isValidValue'));

        ValidationUtil::validate($paymentSetup->getPaymentSchedule(), DataType::OBJECT, Mandatory::YES, $exception, HttpParamConstants::PAYMENT_SCHEDULE, AppErrorCodes::ERROR_SEGMENT_MISSING, CommonUtil::getValidationMethod(__CLASS__, 'validatePaymentSchedule'));
    }

    public function validatePaymentSchedule(PaymentSchedule $paymentSchedule, OrbipayApiException $exception, $segment = '')
    {

        $segment .= HttpParamConstants::DOT;

        ValidationUtil::validate($paymentSchedule->getPaymentRecurringType(), DataType::ENUM, Mandatory::NO, $exception, $segment . HttpParamConstants::PAYMENT_RECURRING_TYPE, AppErrorCodes::PAYMENT_RECURRING_TYPE_INVALID, CommonUtil::getValidationMethod(PaymentRecurringType::class, 'isValidValue'));

        ValidationUtil::validate($paymentSchedule->getPaymentRecurringCount(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::PAYMENT_RECURRING_COUNT, AppErrorCodes::PAYMENT_RECURRING_COUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidPaymentRecurringCount'));

        ValidationUtil::validate($paymentSchedule->getPaymentAmountType(), DataType::ENUM, Mandatory::NO, $exception, $segment . HttpParamConstants::PAYMENT_AMOUNT_TYPE, AppErrorCodes::PAYMENT_AMOUNT_TYPE_INVALID, CommonUtil::getValidationMethod(PaymentAmountType::class, 'isValidValue'));

        ValidationUtil::validate($paymentSchedule->getPaymentStartDate(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::PAYMENT_START_DATE, AppErrorCodes::PAYMENT_START_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));

        ValidationUtil::validate($paymentSchedule->getPaymentEndDate(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::PAYMENT_END_DATE, AppErrorCodes::PAYMENT_END_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));

        ValidationUtil::validate($paymentSchedule->getPaymentLimitAmount(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::PAYMENT_LIMIT_AMOUNT, AppErrorCodes::PAYMENT_LIMIT_AMOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAmount'));

        ValidationUtil::validate($paymentSchedule->getPaymentPlanId(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::PAYMENT_PLAN_ID, AppErrorCodes::PAYMENT_PLAN_ID_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidPaymentPlanId'));

        ValidationUtil::validate($paymentSchedule->getPaymentDeferDays(), DataType::STRING, Mandatory::NO, $exception, $segment . HttpParamConstants::PAYMENT_DEFER_DAYS, AppErrorCodes::PAYMENT_DEFER_DAYS_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidPaymentDeferDays'));
    }


    /**
     * @param array $apiResponse
     * @return PaymentSetup
     */
    private function wrapResponse(array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($paymentSetupResponse, $statusCode, $headers) = $apiResponse;

        $paymentSetup = ResponseWrapper::wrapPaymentSetup($paymentSetupResponse);
        $paymentSetup->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $paymentSetup;
    }

    private function getBody(PaymentSetup $paymentSetup)
    {
        $body = new CreatePaymentSetupRequest();
        $body->setAmount($paymentSetup->getAmount());
        $body->setCardCvvNumber($paymentSetup->getCardCvvNumber());
        $body->setFee(RequestWrapper::getFeeRequest($paymentSetup->getFee()));
        $body->setCustomer(RequestWrapper::getCustomerMinimalRequest($paymentSetup->getCustomer()));
        $body->setCustomerAccount(RequestWrapper::getCustomerAccountMinimalRequest($paymentSetup->getCustomerAccount()));
        $body->setFundingAccount(RequestWrapper::getFundingAccountMinimalRequest($paymentSetup->getFundingAccount()));
        $body->setPaymentSetupReference($paymentSetup->getPaymentSetupReference());
        $body->setPaymentSetupScheduleType($paymentSetup->getPaymentSetupScheduleType() !== null ? $paymentSetup->getPaymentSetupScheduleType()->value() : null);
        $body->setComments($paymentSetup->getComments());
        $body->setPaymentSchedule($this->wrapPaymentSchedule($paymentSetup->getPaymentSchedule()));
        $body->setCustomFields($paymentSetup->getCustomFields());
        return $body;
    }

    private function wrapPaymentSchedule(PaymentSchedule $request = null)
    {
        $paymentScheduleVO = null;
        if ($request != null) {
            $paymentScheduleVO = new PaymentScheduleVO();
            $paymentScheduleVO->setPaymentDeferDays($request->getPaymentDeferDays());
            $paymentScheduleVO->setPaymentAmountType($request->getPaymentAmountType() !== null ? $request->getPaymentAmountType()->value() : null);
            $paymentScheduleVO->setPaymentEndDate($request->getPaymentEndDate());
            $paymentScheduleVO->setPaymentLimitAmount($request->getPaymentLimitAmount());
            $paymentScheduleVO->setPaymentPlanId($request->getPaymentPlanId());
            $paymentScheduleVO->setPaymentRecurringCount($request->getPaymentRecurringCount());
            $paymentScheduleVO->setPaymentRecurringType($request->getPaymentRecurringType() !== null ? $request->getPaymentRecurringType()->value() : null);
            $paymentScheduleVO->setPaymentStartDate($request->getPaymentStartDate());
        }
        return $paymentScheduleVO;
    }
}