<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\PaymentSetup;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\PaymentSetup\PaymentSetup;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\PaymentSetupApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class GetPaymentSetupHandler
{
    /**
     * @param PaymentSetup $paymentSetup
     * @return PaymentSetup
     * @throws OrbipayApiException
     */
    public function process(PaymentSetup $paymentSetup)
    {
        $this->validate($paymentSetup);
        return $this->callAPI($paymentSetup);
    }

    /**
     * @param PaymentSetup $paymentSetup
     * @return PaymentSetup
     * @throws OrbipayApiException
     */
    private function callAPI(PaymentSetup $paymentSetup)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($paymentSetup->getApiEndPoint());
        $configuration->setClientKey($paymentSetup->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($paymentSetup->getInvocationContext()->getSecret());

        $paymentSetupApi = new PaymentSetupApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\PaymentSetupResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $paymentSetupApi->getPaymentSetup($paymentSetup->getInvocationContext()->getChannel(),
                $paymentSetup->getInvocationContext()->getClientKey(),
                $paymentSetup->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $paymentSetup->getInvocationContext()->getIdempotentRequestKey(),
                $paymentSetup->getInvocationContext()->getRequestor(),
                $paymentSetup->getInvocationContext()->getRequestorType()->value(),
                $paymentSetup->getPathParamData()->getIdPaymentSetup(),
                $paymentSetup->getPathParamData()->getIdCustomer(),
                RequestWrapper::wrapXOPAYHeaders($paymentSetup->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $paymentSetup, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $paymentSetup);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $paymentSetup, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    /**
     * @param PaymentSetup $paymentSetup
     * @throws OrbipayApiException
     */
    private function validate(PaymentSetup $paymentSetup)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $paymentSetup);

        ValidationUtil::validate($paymentSetup->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($paymentSetup->getPathParamData()->getIdPaymentSetup(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_PAYMENT_SETUP, AppErrorCodes::ID_PAYMENT_SETUP_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));

        ValidationUtil::validateInvocationContext($paymentSetup->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    /**
     * @param array $apiResponse
     * @return PaymentSetup
     */
    private function wrapResponse(array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($paymentSetupResponse, $statusCode, $headers) = $apiResponse;

        $paymentSetup = ResponseWrapper::wrapPaymentSetup($paymentSetupResponse);
        $paymentSetup->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $paymentSetup;
    }
}