<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\PaymentSetup;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Enum\PaymentSetupScheduleType;
use Orbipay\PaymentsApi\Client\Enum\PaymentSetupStatus;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\PaymentSetup\PaymentSetups;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\PaymentSetupApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class RetrievePaymentSetupsHandler
{
    /**
     * @param PaymentSetups $paymentSetups
     * @return PaymentSetups
     * @throws OrbipayApiException
     */
    public function process(PaymentSetups $paymentSetups)
    {
        $this->validate($paymentSetups);
        return $this->callAPI($paymentSetups);
    }

    /**
     * @param PaymentSetups $paymentSetups
     * @return PaymentSetups
     * @throws OrbipayApiException
     */
    private function callAPI(PaymentSetups $paymentSetups)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($paymentSetups->getApiEndPoint());
        $configuration->setClientKey($paymentSetups->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($paymentSetups->getInvocationContext()->getSecret());

        $paymentSetupApi = new PaymentSetupApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\PaymentSetupsResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $paymentSetupApi->retrievePaymentSetups($paymentSetups->getInvocationContext()->getChannel(),
                $paymentSetups->getInvocationContext()->getClientKey(),
                $paymentSetups->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $paymentSetups->getInvocationContext()->getIdempotentRequestKey(),
                $paymentSetups->getInvocationContext()->getRequestor(),
                $paymentSetups->getInvocationContext()->getRequestorType()->value(),
                RequestWrapper::wrapXOPAYHeaders($paymentSetups->getInvocationContext()->getAttributes()),
                $paymentSetups->getQueryParamData()->getIdCustomer(),
                $paymentSetups->getQueryParamData()->getIdFundingAccount(),
                $paymentSetups->getQueryParamData()->getIdCustomerAccount(),
                $paymentSetups->getQueryParamData()->getConfirmationNumber(),
                $paymentSetups->getQueryParamData()->getStatus(),
                $paymentSetups->getQueryParamData()->getPaymentSetupScheduleType(),
                $paymentSetups->getQueryParamData()->getFromDate(),
                $paymentSetups->getQueryParamData()->getToDate(),
                $paymentSetups->getQueryParamData()->getPageSize(),
                $paymentSetups->getQueryParamData()->getQueryId(),
                $paymentSetups->getQueryParamData()->getToIndex(),
                $paymentSetups->getQueryParamData()->getFromIndex());
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $paymentSetups, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $paymentSetups);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $paymentSetups, $e->getCode(), $e);
        }

        return $this->wrapResponse($paymentSetups, $response);
    }

    /**
     * @param PaymentSetups $paymentSetups
     * @throws OrbipayApiException
     */
    private function validate(PaymentSetups $paymentSetups)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $paymentSetups);

        ValidationUtil::validate($paymentSetups->getQueryParamData()->getIdCustomer(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($paymentSetups->getQueryParamData()->getIdFundingAccount(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_PARAM_ID_FUNDING_ACCOUNT, AppErrorCodes::ID_FUNDING_ACCOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($paymentSetups->getQueryParamData()->getIdCustomerAccount(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_PARAM_ID_CUSTOMER_ACCOUNT, AppErrorCodes::ID_CUSTOMER_ACCOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($paymentSetups->getQueryParamData()->getConfirmationNumber(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::CONFIRMATION_NUMBER, AppErrorCodes::PAYMENT_SETUP_CONFIRMATION_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidConfirmationNumber'));
        ValidationUtil::validate($paymentSetups->getQueryParamData()->getStatus(), DataType::GROUP, Mandatory::NO, $exception, HttpParamConstants::STATUS, AppErrorCodes::QUERY_PARAM_PAYMENT_SETUP_STATUS_INVALID, CommonUtil::getValidationMethod(PaymentSetupStatus::class, 'areValidValues'));
        ValidationUtil::validate($paymentSetups->getQueryParamData()->getPaymentSetupScheduleType(), DataType::GROUP, Mandatory::NO, $exception, HttpParamConstants::PAYMENT_SETUP_SCHEDULE_TYPE, AppErrorCodes::PAYMENT_SETUP_SCHEDULE_TYPE_INVALID, CommonUtil::getValidationMethod(PaymentSetupScheduleType::class, 'areValidValues'));
        ValidationUtil::validate($paymentSetups->getQueryParamData()->getFromDate(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::FROM_DATE, AppErrorCodes::PAYMENT_SETUP_FROM_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));
        ValidationUtil::validate($paymentSetups->getQueryParamData()->getToDate(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::TO_DATE, AppErrorCodes::PAYMENT_SETUP_TO_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));

        ValidationUtil::validate($paymentSetups->getQueryParamData()->getPageSize(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::PAGE_SIZE, AppErrorCodes::PAGE_SIZE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidPageSize'));

        ValidationUtil::validate($paymentSetups->getQueryParamData()->getQueryId(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_ID, AppErrorCodes::QUERY_ID_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidQueryId'));
        ValidationUtil::validate($paymentSetups->getQueryParamData()->getToIndex(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::TO_INDEX, AppErrorCodes::TO_INDEX_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidToIndex'));
        ValidationUtil::validate($paymentSetups->getQueryParamData()->getFromIndex(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::FROM_INDEX, AppErrorCodes::FROM_INDEX_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidFromIndex'));

        ValidationUtil::validateInvocationContext($paymentSetups->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    /**
     * @param PaymentSetups $paymentSetups
     * @param array $apiResponse
     * @return PaymentSetups
     */
    private function wrapResponse(PaymentSetups $paymentSetups, array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($paymentSetupResponse, $statusCode, $headers) = $apiResponse;

        ResponseWrapper::wrapPaymentSetups($paymentSetups, $paymentSetupResponse);
        $paymentSetups->withContext(ResponseWrapper::getInvocationContext($headers, $paymentSetups), $paymentSetups->getInvocationMode(), $paymentSetups->getApiEndPoint());
        return $paymentSetups;
    }
}