<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\Refund;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Enum\RefundType;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\Refund\Refund;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\RefundApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;
use Orbipay\PaymentsApi\Client\Swagger\Model\CreateRefundRequest;

class CreateRefundHandler
{
    /**
     * @param Refund $refund
     * @return Refund
     * @throws OrbipayApiException
     */
    public function process(Refund $refund)
    {
        $this->validate($refund);
        return $this->callAPI($refund);
    }

    /**
     * @param Refund $refund
     * @return Refund
     * @throws OrbipayApiException
     */
    private function callAPI(Refund $refund)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($refund->getApiEndPoint());
        $configuration->setClientKey($refund->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($refund->getInvocationContext()->getSecret());

        $refundApi = new RefundApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\RefundResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $refundApi->createRefund($refund->getInvocationContext()->getChannel(),
                $refund->getInvocationContext()->getClientKey(),
                $refund->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $refund->getInvocationContext()->getIdempotentRequestKey(),
                $refund->getInvocationContext()->getRequestor(),
                $refund->getInvocationContext()->getRequestorType()->value(),
                $refund->getPathParamData()->getIdPayment(),
                $refund->getPathParamData()->getIdCustomer(),
                $this->getBody($refund),
                RequestWrapper::wrapXOPAYHeaders($refund->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $refund, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $refund);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $refund, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    private function getBody(Refund $refund)
    {
        $body = new CreateRefundRequest();
        $body->setRefundAmount($refund->getRefundAmount());
        $body->setRefundReference($refund->getRefundReference());
        $body->setComments($refund->getComments());
        $body->setCustomFields($refund->getCustomFields());
        $body->setFeeRefund(RequestWrapper::getFeeRefundRequest($refund->getFeeRefund()));
        return $body;
    }

    /**
     * @param Refund $refund
     * @throws OrbipayApiException
     */
    private function validate(Refund $refund)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $refund);

        ValidationUtil::validate($refund->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($refund->getPathParamData()->getIdPayment(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_PAYMENT, AppErrorCodes::ID_PAYMENT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));

        $this->validateRefund($refund, $exception);

        ValidationUtil::validateInvocationContext($refund->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    private function validateRefund(Refund $refund, OrbipayApiException $exception)
    {
        ValidationUtil::validate($refund->getRefundAmount(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::REFUND_AMOUNT, AppErrorCodes::AMOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAmount'));

        ValidationUtil::validate($refund->getRefundReference(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::REFUND_REFERENCE, AppErrorCodes::REFUND_REFERENCE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidRefundReference'));

        ValidationUtil::validate($refund->getComments(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::COMMENTS, AppErrorCodes::COMMENTS_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidComments'));
    }

    /**
     * @param array $apiResponse
     * @return Refund
     */
    private function wrapResponse(array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($refundResponse, $statusCode, $headers) = $apiResponse;

        $refund = ResponseWrapper::wrapRefund($refundResponse);
        $refund->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $refund;
    }
}
