<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\Refund;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Enum\PaymentScheduleType;
use Orbipay\PaymentsApi\Client\Enum\PaymentStatus;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\Refund\Refunds;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\RefundApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class RetrieveRefundsHandler
{
    /**
     * @param Refunds $refunds
     * @return Refunds
     * @throws OrbipayApiException
     */
    public function process(Refunds $refunds)
    {
        $this->validate($refunds);
        return $this->callAPI($refunds);
    }


    private function callAPI(Refunds $refunds)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($refunds->getApiEndPoint());
        $configuration->setClientKey($refunds->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($refunds->getInvocationContext()->getSecret());

        $refundApi = new RefundApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\RefundsResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $refundApi->retrieveRefunds($refunds->getInvocationContext()->getChannel(),
                $refunds->getInvocationContext()->getClientKey(),
                $refunds->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $refunds->getInvocationContext()->getIdempotentRequestKey(),
                $refunds->getInvocationContext()->getRequestor(),
                $refunds->getInvocationContext()->getRequestorType()->value(),
                RequestWrapper::wrapXOPAYHeaders($refunds->getInvocationContext()->getAttributes()),
                $refunds->getQueryParamData()->getIdCustomer(),
                $refunds->getQueryParamData()->getConfirmationNumber(),
                $refunds->getQueryParamData()->getStatus(),
                $refunds->getQueryParamData()->getFromDate(),
                $refunds->getQueryParamData()->getToDate(),
                $refunds->getQueryParamData()->getPageSize(),
                $refunds->getQueryParamData()->getQueryId(),
                $refunds->getQueryParamData()->getToIndex(),
                $refunds->getQueryParamData()->getFromIndex());
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $refunds, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $refunds);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $refunds, $e->getCode(), $e);
        }

        return $this->wrapResponse($refunds, $response);
    }

    /**
     * @param Refunds $refunds
     * @throws OrbipayApiException
     */
    private function validate(Refunds $refunds)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $refunds);

        ValidationUtil::validate($refunds->getQueryParamData()->getIdCustomer(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($refunds->getQueryParamData()->getConfirmationNumber(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::CONFIRMATION_NUMBER, AppErrorCodes::PAYMENT_CONFIRMATION_NUMBER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidConfirmationNumber'));
        ValidationUtil::validate($refunds->getQueryParamData()->getStatus(), DataType::GROUP, Mandatory::NO, $exception, HttpParamConstants::STATUS, AppErrorCodes::QUERY_PARAM_PAYMENT_STATUS_INVALID, CommonUtil::getValidationMethod(PaymentStatus::class, 'areValidValues'));
        ValidationUtil::validate($refunds->getQueryParamData()->getFromDate(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::FROM_DATE, AppErrorCodes::PAYMENT_FROM_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));
        ValidationUtil::validate($refunds->getQueryParamData()->getToDate(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::TO_DATE, AppErrorCodes::PAYMENT_TO_DATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidDate'));

        ValidationUtil::validate($refunds->getQueryParamData()->getPageSize(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::PAGE_SIZE, AppErrorCodes::PAGE_SIZE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidPageSize'));

        ValidationUtil::validate($refunds->getQueryParamData()->getQueryId(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::QUERY_ID, AppErrorCodes::QUERY_ID_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidQueryId'));
        ValidationUtil::validate($refunds->getQueryParamData()->getToIndex(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::TO_INDEX, AppErrorCodes::TO_INDEX_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidToIndex'));
        ValidationUtil::validate($refunds->getQueryParamData()->getFromIndex(), DataType::STRING, Mandatory::NO, $exception, HttpParamConstants::FROM_INDEX, AppErrorCodes::FROM_INDEX_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidFromIndex'));


        ValidationUtil::validateInvocationContext($refunds->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    /**
     * @param Refunds $refunds
     * @param array $apiResponse
     * @return Refunds
     */
    private function wrapResponse(Refunds $refunds, array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($refundResponse, $statusCode, $headers) = $apiResponse;

        ResponseWrapper::wrapRefunds($refunds, $refundResponse);
        $refunds->withContext(ResponseWrapper::getInvocationContext($headers, $refunds), $refunds->getInvocationMode(), $refunds->getApiEndPoint());
        return $refunds;
    }
}