<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/20/18
 * Time: 4:17 PM
 */

namespace Orbipay\PaymentsApi\Client\Util;

use DateTimeZone;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Enum;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Enum\Product;
use Orbipay\PaymentsApi\Client\Enum\RequestorType;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\Base\Address;
use Orbipay\PaymentsApi\Client\Model\Base\Attribute;
use Orbipay\PaymentsApi\Client\Model\Base\Attributes;
use Orbipay\PaymentsApi\Client\Model\Base\InvocationContext;
use DateTime;
use Orbipay\PaymentsApi\Client\Model\Customer\Customer;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccount;
use Orbipay\PaymentsApi\Client\Model\FundingAccount\FundingAccount;
use Orbipay\PaymentsApi\Client\Model\Payment\Fee;

class ValidationUtil
{
    public static function isNonEmptyString($data)
    {
        return is_string($data) && $data !== '';
    }

    public static function isNoneOrString($data)
    {
        return $data === null || is_string($data);
    }

    public static function isNoneOrObject($data)
    {
        return $data === null || is_object($data);
    }

    public static function isNoneOrArray($data)
    {
        return $data === null || is_array($data);
    }

    public static function isNoneOrEnum($data)
    {
        return $data === null || $data instanceof Enum;
    }

    public static function isAssociativeArray($arr)
    {
        if (!is_array($arr)) return false;

        if (array() === $arr) return true;
        return array_keys($arr) !== range(0, count($arr) - 1) ? true : false;
    }

    public static function validate($data, $type, $isMandatory, $exception, $fieldName, $appErrorCode, callable $validateFunction = null)
    {
        if (DataType::STRING === $type) {
            self::validateString($data, $isMandatory, $exception, $fieldName, $appErrorCode, $validateFunction);
        } elseif (DataType::OBJECT === $type) {
            self::validateObject($data, $isMandatory, $exception, $fieldName, $appErrorCode, $validateFunction);
        } elseif (DataType::GROUP === $type) {
            self::validateArray($data, $isMandatory, $exception, $fieldName, $appErrorCode, $validateFunction);
        } elseif (DataType::ENUM === $type) {
            self::validateEnum($data, $isMandatory, $exception, $fieldName, $appErrorCode, $validateFunction);
        }
    }

    private static function validateString($data, $isMandatory, OrbipayApiException $exception, $fieldName, $appErrorCode, callable $validateFunction)
    {
        if (!self::isNoneOrString($data)) {
            $exception->addFieldErrorAndResponseCodes($fieldName, $appErrorCode);
            return;
        }

        if (Mandatory::YES == $isMandatory) {
            self::validateMandatoryString($data, $exception, $fieldName, $appErrorCode, $validateFunction);
        } else {
            self::validateOptionalString($data, $exception, $fieldName, $appErrorCode, $validateFunction);
        }

    }

    private static function validateMandatoryString($data, OrbipayApiException $exception, $fieldName, $appErrorCode, callable $validateFunction)
    {
        if ($data === null || $data === '' || ($validateFunction !== null && !call_user_func($validateFunction, $data))) {
            $exception->addFieldErrorAndResponseCodes($fieldName, $appErrorCode);
        }
    }

    private static function validateOptionalString($data, OrbipayApiException $exception, $fieldName, $appErrorCode, callable $validateFunction)
    {
        if ($data !== null && $data !== '' && ($validateFunction !== null && !call_user_func($validateFunction, $data))) {
            $exception->addFieldErrorAndResponseCodes($fieldName, $appErrorCode);
        }
    }

    private static function validateObject($data, $isMandatory, OrbipayApiException $exception, $fieldName, $appErrorCode, callable $validateFunction)
    {
        if (!self::isNoneOrObject($data)) {
            $exception->addFieldErrorAndResponseCodes($fieldName, $appErrorCode);
            return;
        }

        if (Mandatory::YES == $isMandatory) {
            self::validateMandatoryObject($data, $exception, $fieldName, $appErrorCode, $validateFunction);
        } else {
            self::validateOptionalObject($data, $exception, $fieldName, $validateFunction);
        }
    }

    private static function validateMandatoryObject($data, OrbipayApiException $exception, $fieldName, $appErrorCode, callable $validateFunction)
    {
        if ($data === null) {
            $exception->addFieldErrorAndResponseCodes($fieldName, $appErrorCode);
        }

        if ($validateFunction !== null) {
            call_user_func($validateFunction, $data, $exception, $fieldName);
        }
    }

    private static function validateOptionalObject($data, OrbipayApiException $exception, $fieldName, callable $validateFunction)
    {
        if ($data !== null && $validateFunction !== null) {
            call_user_func($validateFunction, $data, $exception, $fieldName);
        }
    }

    private static function validateArray($data, $isMandatory, OrbipayApiException $exception, $fieldName, $appErrorCode, callable $validateFunction)
    {
        if (!self::isNoneOrArray($data)) {
            $exception->addFieldErrorAndResponseCodes($fieldName, $appErrorCode);
            return;
        }

        if (Mandatory::YES == $isMandatory) {
            self::validateMandatoryArray($data, $exception, $fieldName, $appErrorCode, $validateFunction);
        } else {
            self::validateOptionalArray($data, $exception, $fieldName, $appErrorCode, $validateFunction);
        }
    }

    private static function validateMandatoryArray($data, OrbipayApiException $exception, $fieldName, $appErrorCode, callable $validateFunction)
    {
        if ($data === null) {
            $exception->addFieldErrorAndResponseCodes($fieldName, $appErrorCode);
        }

        if ($validateFunction !== null) {
            call_user_func($validateFunction, $data, $exception, $fieldName, $appErrorCode);
        }
    }

    private static function validateOptionalArray($data, OrbipayApiException $exception, $fieldName, $appErrorCode, callable $validateFunction)
    {
        if ($data !== null && $validateFunction !== null) {
            call_user_func($validateFunction, $data, $exception, $fieldName, $appErrorCode);
        }
    }

    private static function validateEnum($data, $isMandatory, OrbipayApiException $exception, $fieldName, $appErrorCode, callable $validateFunction)
    {
        if (!self::isNoneOrEnum($data)) {
            $exception->addFieldErrorAndResponseCodes($fieldName, $appErrorCode);
            return;
        }

        if (Mandatory::YES == $isMandatory) {
            self::validateMandatoryEnum($data, $exception, $fieldName, $appErrorCode, $validateFunction);
        } else {
            self::validateOptionalEnum($data, $exception, $fieldName, $appErrorCode, $validateFunction);
        }
    }

    private static function validateMandatoryEnum($data, OrbipayApiException $exception, $fieldName, $appErrorCode, callable $validateFunction)
    {
        /** @var Enum $data */
        if ($data === null || ($validateFunction !== null && !call_user_func($validateFunction, $data->value()))) {
            $exception->addFieldErrorAndResponseCodes($fieldName, $appErrorCode);
        }
    }

    private static function validateOptionalEnum($data, OrbipayApiException $exception, $fieldName, $appErrorCode, callable $validateFunction)
    {
        /** @var Enum $data */
        if ($data !== null && ($validateFunction !== null && !call_user_func($validateFunction, $data->value()))) {
            $exception->addFieldErrorAndResponseCodes($fieldName, $appErrorCode);
        }
    }
    
    public static function isValidName($data)
    {
        if (preg_match("/^[0-9a-zA-Z #,.''&\/\-@!$%*()_+={}|:;`\[\]\^\~\\\\\"]{0,60}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidFirstName($data)
    {
        return self::isValidName($data);
    }

    public static function isValidLastName($data)
    {
        return self::isValidName($data);
    }

    public static function isValidMiddleName($data)
    {
        return self::isValidName($data);
    }

    public static function isValidDOB($data)
    {
        if (self::isValidDate($data) && !self::isFutureDate($data)) {
            return true;
        }
        return false;
    }

    public static function isValidDate($data)
    {
        if (preg_match("/^[0-9]{4}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])$/", $data)) {
            return true;
        }
        return false;
    }

    /**
     * @param $data
     * @return bool
     */
    public static function isFutureDate($data)
    {
        $dateDefaultTimezone = date_default_timezone_get();
        $now = new DateTime(RestConstants::CURRENT_TIME, new DateTimeZone($dateDefaultTimezone));
        $input_date = DateTime::createFromFormat(RestConstants::DATE_FORMAT, $data, new DateTimeZone($dateDefaultTimezone));
        if ($input_date > $now) {
            return true;
        }
        return false;
    }

    public static function isValidLocale($data)
    {
        if (preg_match("/^[a-z]{2}_[A-Z]{2}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidSSN($data)
    {
        if (preg_match("/^[0-9]{9}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidEmail($data)
    {
        if (strlen($data) > 100 || !preg_match("/^([a-zA-Z0-9!#$%&'*+-\/=?^._`{|}~]+@[a-zA-Z0-9_.-]+\.[a-zA-Z0-9._]+){0,100}$/", $data)) {
            return false;
        }

        return true;
    }

    public static function isValidPhone($data)
    {
        if (preg_match("/^[0-9]{10}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidHomePhone($data)
    {
        return self::isValidPhone($data);
    }

    public static function isValidMobilePhone($data)
    {
        return self::isValidPhone($data);
    }

    public static function isValidWorkPhone($data)
    {
        return self::isValidPhone($data);
    }

    public static function isValidCustomerReference($data)
    {
        if (preg_match("/^[0-9a-zA-Z #,.''&\/\-@!$%*()_+={}|:;`\[\]\^\~\s\\\\\"]{0,100}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidComments($data)
    {
        if (preg_match("/^[0-9a-zA-Z #,.''&\/\-@!$%*()_+={}|:;`\[\]\^\~\s\\\\\"]{0,1000}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidAddressLine($data)
    {
        if (preg_match("/^[0-9a-zA-Z #,.''&\/\-@!$%*()_+={}|:;`\[\]\^\~\\\\\"]{0,100}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidCity($data)
    {
        if (preg_match("/^[0-9a-zA-Z ,.-]{0,50}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidState($data)
    {
        if (preg_match("/^[a-zA-Z]{2}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidCountry($data)
    {
        if (preg_match("/^[a-zA-Z]{3}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidZip1($data)
    {
        if (preg_match("/^[a-zA-Z0-9 -]{2,10}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidZip2($data)
    {
        if (preg_match("/^[0-9]{4}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isFloat($data)
    {
        return ($data == (string)(float)$data);
    }

    public static function isValidCustomerAccountBalance($data)
    {
        if (preg_match("/^(\+|\-)?([0-9]{1,14}(\.[0-9][0-9]?)?)$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidAmount($data)
    {
        if (preg_match("/^([0-9]{1,14}(\.[0-9][0-9]?)?)$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidCustomerAccountReference($data)
    {
        if (preg_match("/^[a-zA-Z0-9._-]{0,50}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidNickname($data)
    {
        if (preg_match("/^[0-9a-zA-Z #,.''&\/\-@!$%*()_+={}|:;`\[\]\^\~\\\\\"]{0,50}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidCustomerAccountNumber($data)
    {
        if (preg_match("/^[a-zA-Z0-9-_\/!@#$%&*{}. ']{1,32}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidAccountHolderName($data)
    {
        return self::isValidName($data);
    }

    public static function isValidFundingAccountNumber($data)
    {
        if (preg_match("/^[a-zA-Z0-9]{4,32}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidExpiryDate($data)
    {
        if (preg_match("/^[0-9]{2}\/[0-9]{2}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidCardCVVNumber($data)
    {
        if (preg_match("/^[0-9]{3,4}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidPaymentReference($data)
    {
        if (preg_match("/^[a-zA-Z0-9._-]{0,50}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidId($data)
    {
        if (preg_match("/^[0-9]{0,20}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidConfirmationNumber($data)
    {
        if (preg_match("/^[a-zA-Z0-9]{0,30}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidPageSize($data)
    {
        if (preg_match("/^[0-9]{0,3}$/", $data)) {
            $int = (int)$data;
            if (1 <= $int && $int <= 100) {
                return true;
            }
        }
        return false;
    }

    public static function isValidFromIndex($data)
    {
        if (preg_match("/^[0-9]{0,4}$/", $data)) {
            $int = (int)$data;
            if (1 <= $int && $int <= 5000) {
                return true;
            }
        }
        return false;
    }

    public static function isValidToIndex($data)
    {
        if (preg_match("/^[0-9]{0,4}$/", $data)) {
            $int = (int)$data;
            if (1 <= $int && $int <= 5000) {
                return true;
            }
        }
        return false;
    }

    public static function isValidQueryId($data)
    {
        if (preg_match("/^[a-zA-Z0-9]{0,50}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidPaymentRecurringCount($data)
    {
        if (preg_match("/^[0-9]{0,3}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidPaymentPlanId($data)
    {
        if (preg_match("/^[0-9]{0,9}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidPaymentDeferDays($data)
    {
        if (preg_match("/^[0-9]{1,2}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidRefundReference($data)
    {
        if (preg_match("/^[a-zA-Z0-9._-]{0,50}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidChannel($data)
    {
        if (strlen($data) <= 100) {
            return true;
        }
        return false;
    }

    public static function isValidClientKey($data)
    {
        if (preg_match("/^[a-zA-Z0-9_-]{1,50}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidIdempotentRequestKey($data)
    {
        if (preg_match("/^[a-zA-Z0-9]{1,50}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidRequestor($data)
    {
        if (preg_match("/^[a-zA-Z0-9]{1,50}$/", $data)) {
            return true;
        }
        return false;
    }

    public static function isValidABARoutingNumber($data)
    {
        if (!preg_match("/^[0-9]{9}$/", $data)) {
            return false;
        }

        $checkSum = 0;
        for ($i = 0, $j = strlen($data); $i < $j; $i += 3) {
            //loop through routingNumber character by character
            $checkSum += ($data[$i] * 3);
            $checkSum += ($data[$i + 1] * 7);
            $checkSum += ($data[$i + 2]);
        }

        if ($checkSum != 0 and ($checkSum % 10) == 0) {
            return true;
        } else {
            return false;
        }
    }

    public static function isValidHeaderParam($data)
    {
        if ($data === HttpParamConstants::CLIENT_KEY || $data === HttpParamConstants::CHANNEL
            || $data === HttpParamConstants::PRODUCT || $data === HttpParamConstants::TIMESTAMP
            || $data === HttpParamConstants::IDEMPOTENT_REQUEST_KEY || $data === HttpParamConstants::REQUESTOR
            || $data === HttpParamConstants::REQUESTOR_TYPE || $data === HttpParamConstants::X_OPAY_HEADERS) {
            return true;
        }
        return false;
    }

    public static function validateCustomerAddress(Address $address, OrbipayApiException $exception, $baseSegment = '')
    {
        if ($address === null || !is_string($baseSegment) || $exception == null) {
            return;
        }

        $baseSegment .= HttpParamConstants::DOT;

        self::validate($address->getAddressLine1(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_LINE1, AppErrorCodes::CUSTOMER_ADDRESS_LINE1_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAddressLine'));
        self::validate($address->getAddressLine2(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_LINE2, AppErrorCodes::CUSTOMER_ADDRESS_LINE2_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAddressLine'));
        self::validate($address->getAddressCity(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_CITY, AppErrorCodes::CUSTOMER_ADDRESS_CITY_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCity'));
        self::validate($address->getAddressState(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_STATE, AppErrorCodes::CUSTOMER_ADDRESS_STATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidState'));
        self::validate($address->getAddressCountry(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_COUNTRY, AppErrorCodes::CUSTOMER_ADDRESS_COUNTRY_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCountry'));
        self::validate($address->getAddressZip1(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_ZIP1, AppErrorCodes::CUSTOMER_ADDRESS_ZIP1_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidZip1'));
        self::validate($address->getAddressZip2(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_ZIP2, AppErrorCodes::CUSTOMER_ADDRESS_ZIP2_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidZip2'));
    }

    public static function validateCustomerAccountAddress(Address $address, OrbipayApiException $exception, $baseSegment = '')
    {
        if ($address === null || !is_string($baseSegment) || $exception == null) {
            return;
        }

        $baseSegment .= HttpParamConstants::DOT;

        self::validate($address->getAddressLine1(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_LINE1, AppErrorCodes::CUSTOMER_ACCOUNT_ADDRESS_LINE1_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAddressLine'));
        self::validate($address->getAddressLine2(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_LINE2, AppErrorCodes::CUSTOMER_ACCOUNT_ADDRESS_LINE2_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAddressLine'));
        self::validate($address->getAddressCity(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_CITY, AppErrorCodes::CUSTOMER_ACCOUNT_ADDRESS_CITY_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCity'));
        self::validate($address->getAddressState(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_STATE, AppErrorCodes::CUSTOMER_ACCOUNT_ADDRESS_STATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidState'));
        self::validate($address->getAddressCountry(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_COUNTRY, AppErrorCodes::CUSTOMER_ACCOUNT_ADDRESS_COUNTRY_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCountry'));
        self::validate($address->getAddressZip1(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_ZIP1, AppErrorCodes::CUSTOMER_ACCOUNT_ADDRESS_ZIP1_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidZip1'));
        self::validate($address->getAddressZip2(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_ZIP2, AppErrorCodes::CUSTOMER_ACCOUNT_ADDRESS_ZIP2_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidZip2'));
    }

    public static function validateFundingAccountAddress(Address $address, OrbipayApiException $exception, $baseSegment = '')
    {

        if ($address === null || !is_string($baseSegment) || $exception == null) {
            return;
        }

        $baseSegment .= HttpParamConstants::DOT;

        self::validate($address->getAddressLine1(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_LINE1, AppErrorCodes::FUNDING_ACCOUNT_ADDRESS_LINE1_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAddressLine'));
        self::validate($address->getAddressLine2(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_LINE2, AppErrorCodes::FUNDING_ACCOUNT_ADDRESS_LINE2_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAddressLine'));
        self::validate($address->getAddressCity(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_CITY, AppErrorCodes::FUNDING_ACCOUNT_ADDRESS_CITY_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCity'));
        self::validate($address->getAddressState(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_STATE, AppErrorCodes::FUNDING_ACCOUNT_ADDRESS_STATE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidState'));
        self::validate($address->getAddressCountry(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_COUNTRY, AppErrorCodes::FUNDING_ACCOUNT_ADDRESS_COUNTRY_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidCountry'));
        self::validate($address->getAddressZip1(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_ZIP1, AppErrorCodes::FUNDING_ACCOUNT_ADDRESS_ZIP1_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidZip1'));
        self::validate($address->getAddressZip2(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::ADDRESS_ZIP2, AppErrorCodes::FUNDING_ACCOUNT_ADDRESS_ZIP2_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidZip2'));
    }

    public static function validateInvocationContext(InvocationContext $invocationContext, OrbipayApiException $exception)
    {

        self::validate($invocationContext->getChannel(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::CHANNEL, AppErrorCodes::CHANNEL_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidChannel'));
        self::validate($invocationContext->getClientKey(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::CLIENT_KEY, AppErrorCodes::CLIENT_KEY_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidClientKey'));
        self::validate($invocationContext->getProduct(), DataType::ENUM, Mandatory::YES, $exception, HttpParamConstants::PRODUCT, AppErrorCodes::PRODUCT_INVALID, CommonUtil::getValidationMethod(Product::class, 'isValidValue'));
        self::validate($invocationContext->getIdempotentRequestKey(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::IDEMPOTENT_REQUEST_KEY, AppErrorCodes::IDEMPOTENT_REQUEST_KEY_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidIdempotentRequestKey'));
        self::validate($invocationContext->getRequestor(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::REQUESTOR, AppErrorCodes::REQUESTOR_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidRequestor'));
        self::validate($invocationContext->getRequestorType(), DataType::ENUM, Mandatory::YES, $exception, HttpParamConstants::REQUESTOR_TYPE, AppErrorCodes::REQUESTOR_TYPE_INVALID, CommonUtil::getValidationMethod(RequestorType::class, 'isValidValue'));
        self::validate($invocationContext->getAttributes(), DataType::OBJECT, Mandatory::NO, $exception, HttpParamConstants::ATTRIBUTES, AppErrorCodes::ATTRIBUTES_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'validateAttributes'));
    }

    public static function validateFee(Fee $fee, OrbipayApiException $exception, $baseSegment = '')
    {
        $baseSegment .= HttpParamConstants::DOT;
        self::validate($fee->getFeeAmount(), DataType::STRING, Mandatory::NO, $exception, $baseSegment . HttpParamConstants::FEE_AMOUNT, AppErrorCodes::CHANNEL_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidAmount'));
    }

    public static function validateCustomer(Customer $customer, OrbipayApiException $exception, $baseSegment = '')
    {
        $baseSegment .= HttpParamConstants::DOT;
        self::validate($customer->getId(), DataType::STRING, Mandatory::YES, $exception, $baseSegment . HttpParamConstants::ID, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
    }

    public static function validateCustomerAccount(CustomerAccount $customerAccount, OrbipayApiException $exception, $baseSegment = '')
    {
        $baseSegment .= HttpParamConstants::DOT;
        self::validate($customerAccount->getId(), DataType::STRING, Mandatory::YES, $exception, $baseSegment . HttpParamConstants::ID, AppErrorCodes::ID_CUSTOMER_ACCOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
    }

    public static function validateFundingAccount(FundingAccount $fundingAccount, OrbipayApiException $exception, $baseSegment = '')
    {
        $baseSegment .= HttpParamConstants::DOT;
        self::validate($fundingAccount->getId(), DataType::STRING, Mandatory::YES, $exception, $baseSegment . HttpParamConstants::ID, AppErrorCodes::EXT_ISSUED_FUNDING_ACCOUNT_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
    }

    public static function validateAttributes(Attributes $attributes, OrbipayApiException $exception, $baseSegment = '')
    {
        $list = $attributes->getList();

        $baseSegment .= HttpParamConstants::DOT;

        $keys = [];
	
	if(is_array($list)){
		/** @var Attribute $attribute */
		foreach ($list as $attribute) {
		    if ($attribute->getKey() === null || !is_string($attribute->getKey()) || $attribute->getKey() === '' || in_array($attribute->getKey(), $keys)) {
		        $exception->addFieldErrorAndResponseCodes($baseSegment . $attribute->getKey(), AppErrorCodes::ATTRIBUTES_INVALID);
		    } else if ($attribute->getValue() === null || !is_string($attribute->getValue()) || $attribute->getValue() === '') {
		        $exception->addFieldErrorAndResponseCodes($baseSegment . $attribute->getKey(), AppErrorCodes::ATTRIBUTES_INVALID);
		    }
		    $keys[] = $attribute->getKey();
		}
	}
    }

}
